@@ -466,6 +466,8 @@ gst_ogg_mux_release_pad (GstElement * element, GstPad * pad)
 
   gst_collect_pads_remove_pad (ogg_mux->collect, pad);
   gst_element_remove_pad (element, pad);
+
+  gst_object_unref (ogg_mux);
 }
 
 /* handle events */
@@ -795,6 +797,10 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 
       /* On EOS we get a NULL buffer */
       if (buf != NULL) {
+        if (ogg_mux->delta_pad == NULL &&
+            GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT))
+          ogg_mux->delta_pad = pad;
+
         incaps = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
         /* if we need headers */
         if (pad->state == GST_OGG_PAD_STATE_CONTROL) {
@@ -1078,11 +1084,17 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GST_LOG_OBJECT (mux, "swapped out page with mime type %s",
         gst_structure_get_name (structure));
 
-    /* quick hack: put theora pages at the front.
+    /* quick hack: put Theora and Dirac video pages at the front.
      * Ideally, we would have a settable enum for which Ogg
-     * profile we work with, and order based on that */
-    if (strcmp (gst_structure_get_name (structure), "video/x-theora") == 0) {
-      GST_DEBUG_OBJECT (thepad, "putting Theora page at the front");
+     * profile we work with, and order based on that.
+     * (FIXME: if there is more than one video stream, shouldn't we only put
+     * one's BOS into the first page, followed by an audio stream's BOS, and
+     * only then followed by the remaining video and audio streams?) */
+    if (gst_structure_has_name (structure, "video/x-theora")) {
+      GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Theora");
+      hbufs = g_list_prepend (hbufs, hbuf);
+    } else if (gst_structure_has_name (structure, "video/x-dirac")) {
+      GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Dirac");
       hbufs = g_list_prepend (hbufs, hbuf);
     } else {
       hbufs = g_list_append (hbufs, hbuf);
@@ -1359,11 +1371,6 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     /* if this is the first packet of a new page figure out the delta flag */
     if (pad->new_page) {
       if (delta_unit) {
-        /* This page is a delta frame */
-        if (ogg_mux->delta_pad == NULL) {
-          /* we got a delta unit on this pad */
-          ogg_mux->delta_pad = pad;
-        }
         /* mark the page as delta */
         pad->first_delta = TRUE;
       } else {
