@@ -114,6 +114,8 @@ GST_ELEMENT_DETAILS ("Text overlay",
 #define DEFAULT_PROP_DELTAY	0
 #define DEFAULT_PROP_WRAP_MODE  GST_TEXT_OVERLAY_WRAP_MODE_WORD_CHAR
 #define DEFAULT_PROP_FONT_DESC	""
+#define DEFAULT_PROP_SILENT	FALSE
+#define DEFAULT_PROP_LINE_ALIGNMENT GST_TEXT_OVERLAY_LINE_ALIGN_CENTER
 
 /* make a property of me */
 #define DEFAULT_SHADING_VALUE    -80
@@ -132,7 +134,9 @@ enum
   PROP_DELTAX,
   PROP_DELTAY,
   PROP_WRAP_MODE,
-  PROP_FONT_DESC
+  PROP_FONT_DESC,
+  PROP_SILENT,
+  PROP_LINE_ALIGNMENT
 };
 
 
@@ -218,6 +222,26 @@ gst_text_overlay_wrap_mode_get_type (void)
   return text_overlay_wrap_mode_type;
 }
 
+#define GST_TYPE_TEXT_OVERLAY_LINE_ALIGN (gst_text_overlay_line_align_get_type())
+static GType
+gst_text_overlay_line_align_get_type (void)
+{
+  static GType text_overlay_line_align_type = 0;
+  static const GEnumValue text_overlay_line_align[] = {
+    {GST_TEXT_OVERLAY_LINE_ALIGN_LEFT, "left", "left"},
+    {GST_TEXT_OVERLAY_LINE_ALIGN_CENTER, "center", "center"},
+    {GST_TEXT_OVERLAY_LINE_ALIGN_RIGHT, "right", "right"},
+    {0, NULL, NULL}
+  };
+
+  if (!text_overlay_line_align_type) {
+    text_overlay_line_align_type =
+        g_enum_register_static ("GstTextOverlayLineAlign",
+        text_overlay_line_align);
+  }
+  return text_overlay_line_align_type;
+}
+
 /* These macros are adapted from videotestsrc.c */
 #define I420_Y_ROWSTRIDE(width) (GST_ROUND_UP_4(width))
 #define I420_U_ROWSTRIDE(width) (GST_ROUND_UP_8(width)/2)
@@ -356,6 +380,30 @@ gst_text_overlay_class_init (GstTextOverlayClass * klass)
           "Pango font description of font to be used for rendering. "
           "See documentation of pango_font_description_from_string "
           "for syntax.", DEFAULT_PROP_FONT_DESC, G_PARAM_WRITABLE));
+  /**
+   * GstTextOverlay:line-alignment
+   *
+   * Alignment of text lines relative to each other (for multi-line text)
+   *
+   * Since: 0.10.15
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LINE_ALIGNMENT,
+      g_param_spec_enum ("line-alignment", "line alignment",
+          "Alignment of text lines relative to each other.",
+          GST_TYPE_TEXT_OVERLAY_LINE_ALIGN, DEFAULT_PROP_LINE_ALIGNMENT,
+          G_PARAM_READWRITE));
+  /**
+   * GstTextOverlay:silent
+   *
+   * If set, no text is rendered. Useful to switch off text rendering
+   * temporarily without removing the textoverlay element from the pipeline.
+   *
+   * Since: 0.10.15
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SILENT,
+      g_param_spec_boolean ("silent", "silent",
+          "Whether to render the text string",
+          DEFAULT_PROP_SILENT, G_PARAM_READWRITE));
 }
 
 static void
@@ -436,9 +484,11 @@ gst_text_overlay_init (GstTextOverlay * overlay, GstTextOverlayClass * klass)
       GST_DEBUG_FUNCPTR (gst_text_overlay_src_event));
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->srcpad);
 
+  overlay->line_align = DEFAULT_PROP_LINE_ALIGNMENT;
   overlay->layout =
       pango_layout_new (GST_TEXT_OVERLAY_GET_CLASS (overlay)->pango_context);
-  pango_layout_set_alignment (overlay->layout, PANGO_ALIGN_CENTER);
+  pango_layout_set_alignment (overlay->layout,
+      (PangoAlignment) overlay->line_align);
   memset (&overlay->bitmap, 0, sizeof (overlay->bitmap));
 
   overlay->halign = DEFAULT_PROP_HALIGNMENT;
@@ -452,6 +502,7 @@ gst_text_overlay_init (GstTextOverlay * overlay, GstTextOverlayClass * klass)
 
   overlay->want_shading = DEFAULT_PROP_SHADING;
   overlay->shading_value = DEFAULT_SHADING_VALUE;
+  overlay->silent = DEFAULT_PROP_SILENT;
 
   overlay->default_text = g_strdup (DEFAULT_PROP_TEXT);
   overlay->need_render = TRUE;
@@ -630,6 +681,14 @@ gst_text_overlay_set_property (GObject * object, guint prop_id,
       }
       break;
     }
+    case PROP_SILENT:
+      overlay->silent = g_value_get_boolean (value);
+      break;
+    case PROP_LINE_ALIGNMENT:
+      overlay->line_align = g_value_get_enum (value);
+      pango_layout_set_alignment (overlay->layout,
+          (PangoAlignment) overlay->line_align);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -674,6 +733,12 @@ gst_text_overlay_get_property (GObject * object, guint prop_id,
     case PROP_WRAP_MODE:
       g_value_set_enum (value, overlay->wrap_mode);
       break;
+    case PROP_SILENT:
+      g_value_set_boolean (value, overlay->silent);
+      break;
+    case PROP_LINE_ALIGNMENT:
+      g_value_set_enum (value, overlay->line_align);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1092,7 +1157,11 @@ gst_text_overlay_text_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (overlay, "TEXT SEGMENT now: %" GST_SEGMENT_FORMAT,
             &overlay->text_segment);
         GST_OBJECT_UNLOCK (overlay);
+      } else {
+        GST_ELEMENT_WARNING (overlay, STREAM, MUX, (NULL),
+            ("received non-TIME newsegment event on text input"));
       }
+
       gst_event_unref (event);
       ret = TRUE;
 
@@ -1164,9 +1233,16 @@ gst_text_overlay_video_event (GstPad * pad, GstEvent * event)
       gst_event_parse_new_segment (event, &update, &rate, &format, &start,
           &stop, &time);
 
-      /* now copy over the values */
-      gst_segment_set_newsegment (overlay->segment, update, rate, format,
-          start, stop, time);
+      if (format == GST_FORMAT_TIME) {
+        GST_DEBUG_OBJECT (overlay, "VIDEO SEGMENT now: %" GST_SEGMENT_FORMAT,
+            overlay->segment);
+
+        gst_segment_set_newsegment (overlay->segment, update, rate, format,
+            start, stop, time);
+      } else {
+        GST_ELEMENT_WARNING (overlay, STREAM, MUX, (NULL),
+            ("received non-TIME newsegment event on video input"));
+      }
 
       ret = gst_pad_event_default (pad, event);
       break;
@@ -1194,6 +1270,7 @@ gst_text_overlay_video_event (GstPad * pad, GstEvent * event)
       break;
     default:
       ret = gst_pad_event_default (pad, event);
+      break;
   }
 
   gst_object_unref (overlay);
@@ -1373,6 +1450,16 @@ wait_for_text_buf:
   if (overlay->video_flushing)
     goto flushing;
 
+  if (overlay->silent) {
+    GST_OBJECT_UNLOCK (overlay);
+    ret = gst_pad_push (overlay->srcpad, buffer);
+
+    /* Update last_stop */
+    gst_segment_set_last_stop (overlay->segment, GST_FORMAT_TIME, clip_start);
+
+    return ret;
+  }
+
   /* Text pad not linked, rendering internal text */
   if (!overlay->text_linked) {
     if (klass->get_text) {
