@@ -802,63 +802,16 @@ copy_samples (float *out, float **in, guint samples, gint channels)
 #endif
 }
 
-/* clip output samples to the segment boundaries
- */
-static gboolean
-vorbis_do_clip (GstVorbisDec * dec, GstBuffer * buf)
-{
-  gint64 start, stop, cstart, cstop, diff;
-
-  start = GST_BUFFER_TIMESTAMP (buf);
-  stop = start + GST_BUFFER_DURATION (buf);
-
-  if (!gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
-          start, stop, &cstart, &cstop))
-    goto clipped;
-
-  /* see if some clipping happened */
-  diff = cstart - start;
-  if (diff > 0) {
-    GST_BUFFER_TIMESTAMP (buf) = cstart;
-    GST_BUFFER_DURATION (buf) -= diff;
-
-    /* bring clipped time to samples */
-    diff = gst_util_uint64_scale_int (diff, dec->vi.rate, GST_SECOND);
-    /* samples to bytes */
-    diff *= (sizeof (float) * dec->vi.channels);
-    GST_DEBUG_OBJECT (dec, "clipping start to %" GST_TIME_FORMAT " %"
-        G_GUINT64_FORMAT " bytes", GST_TIME_ARGS (cstart), diff);
-    GST_BUFFER_DATA (buf) += diff;
-    GST_BUFFER_SIZE (buf) -= diff;
-  }
-  diff = stop - cstop;
-  if (diff > 0) {
-    GST_BUFFER_DURATION (buf) -= diff;
-
-    /* bring clipped time to samples and then to bytes */
-    diff = gst_util_uint64_scale_int (diff, dec->vi.rate, GST_SECOND);
-    diff *= (sizeof (float) * dec->vi.channels);
-    GST_DEBUG_OBJECT (dec, "clipping stop to %" GST_TIME_FORMAT " %"
-        G_GUINT64_FORMAT " bytes", GST_TIME_ARGS (cstop), diff);
-    GST_BUFFER_SIZE (buf) -= diff;
-  }
-
-  return FALSE;
-
-  /* dropped buffer */
-clipped:
-  {
-    GST_DEBUG_OBJECT (dec, "clipped buffer");
-    gst_buffer_unref (buf);
-    return TRUE;
-  }
-}
-
 static GstFlowReturn
 vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
 {
   GstFlowReturn result;
-  gint64 outoffset = GST_BUFFER_OFFSET (buf);
+  gint64 outoffset, origoffset;
+
+  origoffset = GST_BUFFER_OFFSET (buf);
+
+again:
+  outoffset = origoffset;
 
   if (outoffset == -1) {
     dec->queued = g_list_append (dec->queued, buf);
@@ -866,32 +819,54 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
     result = GST_FLOW_OK;
   } else {
     if (G_UNLIKELY (dec->queued)) {
-      gint64 size;
+      guint size;
+      GstClockTime ts;
       GList *walk;
 
       GST_DEBUG_OBJECT (dec, "first buffer with offset %lld", outoffset);
+      ts = gst_util_uint64_scale_int (outoffset, GST_SECOND, dec->vi.rate);
 
       size = g_list_length (dec->queued);
+      /* we walk the queued up list in reverse, and set the buffer fields
+       * calculating backwards */
       for (walk = g_list_last (dec->queued); walk;
           walk = g_list_previous (walk)) {
         GstBuffer *buffer = GST_BUFFER (walk->data);
+        guint offset;
 
-        outoffset -=
-            GST_BUFFER_SIZE (buffer) / (sizeof (float) * dec->vi.channels);
+        offset = GST_BUFFER_SIZE (buffer) / (sizeof (float) * dec->vi.channels);
+
+        if (outoffset >= offset)
+          outoffset -= offset;
+        else {
+          /* we can't go below 0, this means this first offset was at the eos
+           * page and we need to clip to it instead */
+          GST_DEBUG_OBJECT (dec, "clipping %" G_GINT64_FORMAT,
+              offset - outoffset);
+          origoffset += (offset - outoffset);
+          goto again;
+        }
 
         GST_BUFFER_OFFSET (buffer) = outoffset;
         GST_BUFFER_TIMESTAMP (buffer) =
             gst_util_uint64_scale_int (outoffset, GST_SECOND, dec->vi.rate);
-        GST_DEBUG_OBJECT (dec, "patch buffer %" G_GUINT64_FORMAT
-            " offset %" G_GUINT64_FORMAT, size, outoffset);
+        GST_BUFFER_DURATION (buffer) = GST_CLOCK_DIFF (GST_BUFFER_TIMESTAMP
+            (buffer), ts);
+        ts = GST_BUFFER_TIMESTAMP (buffer);
+        GST_DEBUG_OBJECT (dec, "patch buffer %u, offset %" G_GUINT64_FORMAT
+            ", timestamp %" GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+            size, outoffset,
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
+            GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
         size--;
       }
       for (walk = dec->queued; walk; walk = g_list_next (walk)) {
         GstBuffer *buffer = GST_BUFFER (walk->data);
 
-        /* clips or returns FALSE with buffer unreffed when completely
-         * clipped */
-        if (vorbis_do_clip (dec, buffer))
+        /* clips to the configured segment, or returns NULL with buffer
+         * unreffed when the input buffer is completely outside the segment */
+        if (!(buffer = gst_audio_buffer_clip (buffer, &dec->segment,
+                    dec->vi.rate, dec->vi.channels * sizeof (float))))
           continue;
 
         if (dec->discont) {
@@ -906,7 +881,8 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
     }
 
     /* clip */
-    if (vorbis_do_clip (dec, buf))
+    if (!(buf = gst_audio_buffer_clip (buf, &dec->segment, dec->vi.rate,
+                dec->vi.channels * sizeof (float))))
       return GST_FLOW_OK;
 
     if (dec->discont) {
@@ -937,7 +913,7 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
   guint sample_count;
   GstBuffer *out;
   GstFlowReturn result;
-  GstClockTime timestamp = -1;
+  GstClockTime timestamp = GST_CLOCK_TIME_NONE, nextts;
   gint size;
 
   if (!vd->initialized)
@@ -970,6 +946,7 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
   if ((sample_count = vorbis_synthesis_pcmout (&vd->vd, NULL)) == 0)
     goto done;
 
+  GST_LOG_OBJECT (vd, "%d samples ready for reading", sample_count);
   size = sample_count * vd->vi.channels * sizeof (float);
 
   /* alloc buffer for it */
@@ -996,7 +973,7 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
     vd->granulepos = packet->granulepos - sample_count;
 
   if (vd->cur_timestamp != GST_CLOCK_TIME_NONE) {
-    /* we have incomming timestamps */
+    /* we have incoming timestamps */
     timestamp = vd->cur_timestamp;
     GST_DEBUG_OBJECT (vd,
         "cur_timestamp: %" GST_TIME_FORMAT " + %" GST_TIME_FORMAT " = %"
@@ -1008,12 +985,22 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
         vd->vi.rate);
     GST_BUFFER_OFFSET_END (out) = GST_BUFFER_OFFSET (out) + sample_count;
   } else {
-    /* we have incomming granulepos */
+    /* we have incoming granulepos */
     GST_BUFFER_OFFSET (out) = vd->granulepos;
     if (vd->granulepos != -1) {
+      GST_DEBUG_OBJECT (vd, "granulepos: %" G_GINT64_FORMAT, vd->granulepos);
       GST_BUFFER_OFFSET_END (out) = vd->granulepos + sample_count;
       timestamp =
           gst_util_uint64_scale_int (vd->granulepos, GST_SECOND, vd->vi.rate);
+      nextts =
+          gst_util_uint64_scale_int (vd->granulepos + sample_count,
+          GST_SECOND, vd->vi.rate);
+      GST_DEBUG_OBJECT (vd, "corresponding timestamp %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (timestamp));
+      /* calculate a nano-second accurate duration */
+      GST_BUFFER_DURATION (out) = GST_CLOCK_DIFF (timestamp, nextts);
+      GST_DEBUG_OBJECT (vd, "set duration %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_DURATION (out)));
     } else {
       timestamp = -1;
     }
@@ -1032,7 +1019,7 @@ done:
   vorbis_synthesis_read (&vd->vd, sample_count);
 
   GST_DEBUG_OBJECT (vd,
-      "decoded %d bytes into %d samples, ts %" GST_TIME_FORMAT, packet->bytes,
+      "decoded %ld bytes into %d samples, ts %" GST_TIME_FORMAT, packet->bytes,
       sample_count, GST_TIME_ARGS (timestamp));
 
   /* granulepos is the last sample in the packet */
@@ -1106,8 +1093,13 @@ vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
    */
   packet.e_o_s = 0;
 
-  if (G_UNLIKELY (packet.bytes < 1))
-    goto wrong_size;
+  /* error out on empty header packets, but just skip empty data packets */
+  if (G_UNLIKELY (packet.bytes == 0)) {
+    if (vd->initialized)
+      goto empty_buffer;
+    else
+      goto empty_header;
+  }
 
   GST_DEBUG_OBJECT (vd, "vorbis granule: %" G_GINT64_FORMAT,
       (gint64) packet.granulepos);
@@ -1126,10 +1118,21 @@ vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
 done:
   return result;
 
-  /* ERRORS */
-wrong_size:
+empty_buffer:
   {
-    GST_ELEMENT_ERROR (vd, STREAM, DECODE, (NULL), ("empty buffer received"));
+    /* don't error out here, just ignore the buffer, it's invalid for vorbis
+     * but not fatal. */
+    GST_WARNING_OBJECT (vd, "empty buffer received, ignoring");
+    if (packet.granulepos != -1)
+      vd->granulepos = packet.granulepos;
+    result = GST_FLOW_OK;
+    goto done;
+  }
+
+/* ERRORS */
+empty_header:
+  {
+    GST_ELEMENT_ERROR (vd, STREAM, DECODE, (NULL), ("empty header received"));
     result = GST_FLOW_ERROR;
     vd->discont = TRUE;
     goto done;
@@ -1155,7 +1158,7 @@ wrong_size:
  *    output queue:    
  *
  * When a DISCONT is received (buffer 4), we move the gather queue to the 
- * decode queue. This simply done be taking the head of the gather queue
+ * decode queue. This is simply done be taking the head of the gather queue
  * and prepending it to the decode queue. This yields:
  * 
  *    gather queue:    
@@ -1169,7 +1172,7 @@ wrong_size:
  *
  *    gather queue:    
  *    decode queue:    7  8  9
- *    output queue:    8  9
+ *    output queue:    9  8
  *
  * Then we remove the consumed buffers from the decode queue. Buffer 7 is not
  * completely consumed, we need to keep it around for when we receive buffer 
@@ -1260,7 +1263,8 @@ vorbis_dec_flush_decode (GstVorbisDec * dec)
       GST_BUFFER_DURATION (buf) = endts - GST_BUFFER_TIMESTAMP (buf);
 
       /* clip, this will unref the buffer in case of clipping */
-      if (vorbis_do_clip (dec, buf)) {
+      if (!(buf = gst_audio_buffer_clip (buf, &dec->segment, dec->vi.rate,
+                  dec->vi.channels * sizeof (float)))) {
         GST_DEBUG_OBJECT (dec, "clipped buffer %p", buf);
         goto next;
       }
