@@ -88,7 +88,7 @@
  * </para>
  * <para>
  * The application will then call gst_install_plugins_async(), passing a
- * NULL-terminated array of installer detail strings, and a function that
+ * #NULL-terminated array of installer detail strings, and a function that
  * should be called when the installation of the plugins has finished
  * (successfully or not). Optionally, a #GstInstallPluginsContext created
  * with gst_install_plugins_context_new() may be passed as well. This way
@@ -144,7 +144,7 @@
  * as interlocutor to the real installer though, even more so if command line
  * argument munging is required to transform the command line arguments
  * passed by GStreamer to the helper application into arguments that are
- * understood by the real installer.
+ * understood by the reeal installer.
  * </para>
  * <para>
  * The helper application path defined at compile time can be overriden at
@@ -366,8 +366,6 @@
 #include <sys/wait.h>
 #endif
 
-#include <string.h>
-
 /* best effort to make things compile and possibly even work on win32 */
 #ifndef WEXITSTATUS
 # define WEXITSTATUS(status) ((((guint)(status)) & 0xff00) >> 8)
@@ -391,9 +389,8 @@ struct _GstInstallPluginsContext
  *
  * This function is for X11-based applications (such as most Gtk/Qt
  * applications on linux/unix) only. You can use it to tell the external
- * installer the XID of your main application window. That way the installer
- * can make its own window transient to your application window during the
- * installation.
+ * the XID of your main application window, so the installer can make its
+ * own window transient to your application windonw during the installation.
  *
  * If set, the XID will be passed to the installer via a --transient-for=XID
  * command line option.
@@ -454,30 +451,6 @@ gst_install_plugins_context_free (GstInstallPluginsContext * ctx)
   g_free (ctx);
 }
 
-static GstInstallPluginsContext *
-gst_install_plugins_context_copy (GstInstallPluginsContext * ctx)
-{
-  GstInstallPluginsContext *ret;
-
-  ret = gst_install_plugins_context_new ();
-  ret->xid = ctx->xid;
-
-  return ret;
-}
-
-GType
-gst_install_plugins_context_get_type (void)
-{
-  static GType gst_ipc_type = 0;
-
-  if (G_UNLIKELY (gst_ipc_type == 0)) {
-    gst_ipc_type = g_boxed_type_register_static ("GstInstallPluginsContext",
-        (GBoxedCopyFunc) gst_install_plugins_context_copy,
-        (GBoxedFreeFunc) gst_install_plugins_context_free);
-  }
-  return gst_ipc_type;
-}
-
 static const gchar *
 gst_install_plugins_get_helper (void)
 {
@@ -491,18 +464,6 @@ gst_install_plugins_get_helper (void)
   return helper;
 }
 
-static gboolean
-ptr_array_contains_string (GPtrArray * arr, const gchar * s)
-{
-  gint i;
-
-  for (i = 0; i < arr->len; ++i) {
-    if (strcmp ((const char *) g_ptr_array_index (arr, i), s) == 0)
-      return TRUE;
-  }
-  return FALSE;
-}
-
 static gboolean
 gst_install_plugins_spawn_child (gchar ** details,
     GstInstallPluginsContext * ctx, GPid * child_pid, gint * exit_status)
@@ -523,10 +484,9 @@ gst_install_plugins_spawn_child (gchar ** details,
     g_ptr_array_add (arr, xid_str);
   }
 
-  /* finally, add the detail strings, but without duplicates */
+  /* finally, add the detail strings */
   while (details != NULL && details[0] != NULL) {
-    if (!ptr_array_contains_string (arr, details[0]))
-      g_ptr_array_add (arr, details[0]);
+    g_ptr_array_add (arr, details[0]);
     ++details;
   }
 
@@ -545,7 +505,7 @@ gst_install_plugins_spawn_child (gchar ** details,
     ret = g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
         NULL, child_pid, &err);
   } else {
-    g_return_val_if_reached (FALSE);
+    g_assert_not_reached ();
   }
 
   if (!ret) {
@@ -608,8 +568,8 @@ gst_install_plugins_installer_exited (GPid pid, gint status, gpointer data)
  * gst_install_plugins_async:
  * @details: NULL-terminated array of installer string details (see below)
  * @ctx: a #GstInstallPluginsContext, or NULL
- * @func: (scope async): the function to call when the installer program returns
- * @user_data: (closure): the user data to pass to @func when called, or NULL
+ * @func: the function to call when the installer program returns
+ * @user_data: the user data to pass to @func when called, or NULL
  * 
  * Requests plugin installation without blocking. Once the plugins have been
  * installed or installation has failed, @func will be called with the result
