@@ -21,7 +21,7 @@
 /**
  * SECTION:gstrtpbuffer
  * @short_description: Helper methods for dealing with RTP buffers
- * @see_also: gstbasertppayload, gstbasertpdepayload
+ * @see_also: #GstBaseRTPPayload, #GstBaseRTPDepayload, gstrtcpbuffer
  *
  * <refsect2>
  * <para>
@@ -36,8 +36,14 @@
 
 #include "gstrtpbuffer.h"
 
+#include <stdlib.h>
+#include <string.h>
+
 #define GST_RTP_HEADER_LEN 12
 
+/* Note: we use bitfields here to make sure the compiler doesn't add padding
+ * between fields on certain architectures; can't assume aligned access either
+ */
 typedef struct _GstRTPHeader
 {
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -57,10 +63,10 @@ typedef struct _GstRTPHeader
 #else
 #error "G_BYTE_ORDER should be big or little endian."
 #endif
-  guint16 seq;                  /* sequence number */
-  guint32 timestamp;            /* timestamp */
-  guint32 ssrc;                 /* synchronization source */
-  guint32 csrc[1];              /* optional CSRC list */
+  unsigned int seq:16;          /* sequence number */
+  unsigned int timestamp:32;    /* timestamp */
+  unsigned int ssrc:32;         /* synchronization source */
+  guint8 csrclist[4];           /* optional CSRC list, 32 bits each */
 } GstRTPHeader;
 
 #define GST_RTP_HEADER_VERSION(buf)     (((GstRTPHeader *)(GST_BUFFER_DATA (buf)))->version)
@@ -72,8 +78,10 @@ typedef struct _GstRTPHeader
 #define GST_RTP_HEADER_SEQ(buf)         (((GstRTPHeader *)(GST_BUFFER_DATA (buf)))->seq)
 #define GST_RTP_HEADER_TIMESTAMP(buf)   (((GstRTPHeader *)(GST_BUFFER_DATA (buf)))->timestamp)
 #define GST_RTP_HEADER_SSRC(buf)        (((GstRTPHeader *)(GST_BUFFER_DATA (buf)))->ssrc)
-#define GST_RTP_HEADER_CSRC(buf,i)      (((GstRTPHeader *)(GST_BUFFER_DATA (buf)))->csrc[i])
-
+#define GST_RTP_HEADER_CSRC_LIST_OFFSET(buf,i)  \
+    GST_BUFFER_DATA (buf) +                     \
+    G_STRUCT_OFFSET(GstRTPHeader, csrclist) +   \
+    ((i) * sizeof(guint32))
 #define GST_RTP_HEADER_CSRC_SIZE(buf)   (GST_RTP_HEADER_CSRC_COUNT(buf) * sizeof (guint32))
 
 /**
@@ -108,7 +116,9 @@ gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
   GST_RTP_HEADER_VERSION (buffer) = GST_RTP_VERSION;
   GST_RTP_HEADER_PADDING (buffer) = FALSE;
   GST_RTP_HEADER_EXTENSION (buffer) = FALSE;
-  GST_RTP_HEADER_CSRC_COUNT (buffer) = 0;
+  GST_RTP_HEADER_CSRC_COUNT (buffer) = csrc_count;
+  memset (GST_RTP_HEADER_CSRC_LIST_OFFSET (buffer, 0), 0,
+      csrc_count * sizeof (guint32));
   GST_RTP_HEADER_MARKER (buffer) = FALSE;
   GST_RTP_HEADER_PAYLOAD_TYPE (buffer) = 0;
   GST_RTP_HEADER_SEQ (buffer) = 0;
@@ -308,14 +318,35 @@ gst_rtp_buffer_validate_data (guint8 * data, guint len)
   csrc_count = (data[0] & 0x0f);
   header_len += csrc_count * sizeof (guint32);
 
+  /* calc extension length when present. */
+  if (data[0] & 0x10) {
+    guint8 *extpos;
+    guint16 extlen;
+
+    /* this points to the extenstion bits and header length */
+    extpos = &data[header_len];
+
+    /* skip the header and check that we have enough space */
+    header_len += 4;
+    if (G_UNLIKELY (len < header_len))
+      goto wrong_length;
+
+    /* skip id */
+    extpos += 2;
+    /* read length as the number of 32 bits words */
+    extlen = GST_READ_UINT16_BE (extpos);
+
+    header_len += extlen * sizeof (guint32);
+  }
+
   /* check for padding */
-  if (data[0] & 0x40)
+  if (data[0] & 0x20)
     padding = data[len - 1];
   else
     padding = 0;
 
   /* check if padding not bigger than packet and header */
-  if (G_UNLIKELY (len - header_len <= padding))
+  if (G_UNLIKELY (len - header_len < padding))
     goto wrong_padding;
 
   return TRUE;
@@ -333,8 +364,7 @@ wrong_version:
   }
 wrong_padding:
   {
-    GST_DEBUG ("padding check failed (%d - %d <= %d)",
-        len, header_len, padding);
+    GST_DEBUG ("padding check failed (%d - %d < %d)", len, header_len, padding);
     return FALSE;
   }
 }
@@ -408,6 +438,29 @@ gst_rtp_buffer_get_packet_len (GstBuffer * buffer)
   return GST_BUFFER_SIZE (buffer);
 }
 
+/**
+ * gst_rtp_buffer_get_header_len:
+ * @buffer: the buffer
+ *
+ * Return the total length of the header in @buffer. This include the length of
+ * the fixed header, the CSRC list and the extension header.
+ *
+ * Returns: The total length of the header in @buffer.
+ */
+guint
+gst_rtp_buffer_get_header_len (GstBuffer * buffer)
+{
+  guint len;
+
+  g_return_val_if_fail (GST_IS_BUFFER (buffer), 0);
+
+  len = GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (buffer);
+  if (GST_RTP_HEADER_EXTENSION (buffer))
+    len += GST_READ_UINT16_BE (GST_BUFFER_DATA (buffer) + len + 2) * 4 + 4;
+
+  return len;
+}
+
 /**
  * gst_rtp_buffer_get_version:
  * @buffer: the buffer
@@ -480,7 +533,7 @@ gst_rtp_buffer_set_padding (GstBuffer * buffer, gboolean padding)
  * @buffer: the buffer
  * @len: the new amount of padding
  *
- * Set the amount of padding in the RTP packing in @buffer to
+ * Set the amount of padding in the RTP packet in @buffer to
  * @len. If @len is 0, the padding is removed.
  *
  * NOTE: This function does not work correctly.
@@ -532,6 +585,51 @@ gst_rtp_buffer_set_extension (GstBuffer * buffer, gboolean extension)
   GST_RTP_HEADER_EXTENSION (buffer) = extension;
 }
 
+/**
+ * gst_rtp_buffer_get_extension_data:
+ * @buffer: the buffer
+ * @bits: location for result bits
+ * @data: location for data
+ * @wordlen: location for length of @data in 32 bits words
+ *
+ * Get the extension data. @bits will contain the extrnsion 16 bits of custom
+ * data. @data will point to the data in the extension and @wordlen will contain
+ * the length of @data in 32 bits words.
+ *
+ * If @buffer did not contain an extenstion, this function will return %FALSE
+ * with @bits, @data and @wordlen unchanged.
+ * 
+ * Returns: TRUE if @buffer had the extension bit set.
+ *
+ * Since: 0.10.15
+ */
+gboolean
+gst_rtp_buffer_get_extension_data (GstBuffer * buffer, guint16 * bits,
+    gpointer * data, guint * wordlen)
+{
+  guint len;
+  guint8 *pdata;
+
+  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
+  g_return_val_if_fail (GST_BUFFER_DATA (buffer) != NULL, FALSE);
+
+  if (GST_RTP_HEADER_EXTENSION (buffer))
+    return FALSE;
+
+  /* move to the extension */
+  len = GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (buffer);
+  pdata = GST_BUFFER_DATA (buffer) + len;
+
+  if (bits)
+    *bits = GST_READ_UINT16_BE (pdata);
+  if (wordlen)
+    *wordlen = GST_READ_UINT16_BE (pdata + 2);
+  if (data)
+    *data = pdata + 4;
+
+  return TRUE;
+}
+
 /**
  * gst_rtp_buffer_get_ssrc:
  * @buffer: the buffer
@@ -596,9 +694,9 @@ gst_rtp_buffer_get_csrc (GstBuffer * buffer, guint8 idx)
 {
   g_return_val_if_fail (GST_IS_BUFFER (buffer), 0);
   g_return_val_if_fail (GST_BUFFER_DATA (buffer) != NULL, 0);
-  g_return_val_if_fail (GST_RTP_HEADER_CSRC_COUNT (buffer) < idx, 0);
+  g_return_val_if_fail (idx < GST_RTP_HEADER_CSRC_COUNT (buffer), 0);
 
-  return g_ntohl (GST_RTP_HEADER_CSRC (buffer, idx));
+  return GST_READ_UINT32_BE (GST_RTP_HEADER_CSRC_LIST_OFFSET (buffer, idx));
 }
 
 /**
@@ -614,9 +712,9 @@ gst_rtp_buffer_set_csrc (GstBuffer * buffer, guint8 idx, guint32 csrc)
 {
   g_return_if_fail (GST_IS_BUFFER (buffer));
   g_return_if_fail (GST_BUFFER_DATA (buffer) != NULL);
-  g_return_if_fail (GST_RTP_HEADER_CSRC_COUNT (buffer) < idx);
+  g_return_if_fail (idx < GST_RTP_HEADER_CSRC_COUNT (buffer));
 
-  GST_RTP_HEADER_CSRC (buffer, idx) = g_htonl (csrc);
+  GST_WRITE_UINT32_BE (GST_RTP_HEADER_CSRC_LIST_OFFSET (buffer, idx), csrc);
 }
 
 /**
@@ -777,13 +875,16 @@ gst_rtp_buffer_get_payload_subbuffer (GstBuffer * buffer, guint offset,
 
   plen = gst_rtp_buffer_get_payload_len (buffer);
   /* we can't go past the length */
-  g_return_val_if_fail (offset < plen, NULL);
+  if (G_UNLIKELY (offset >= plen)) {
+    GST_WARNING ("offset=%u should be less then plen=%u", offset, plen);
+    return (NULL);
+  }
 
   /* apply offset */
-  poffset = GST_RTP_HEADER_LEN + GST_RTP_HEADER_CSRC_SIZE (buffer) + offset;
+  poffset = gst_rtp_buffer_get_header_len (buffer) + offset;
   plen -= offset;
 
-  /* see if we need to shrink the buffer even based on @len */
+  /* see if we need to shrink the buffer based on @len */
   if (len != -1 && len < plen)
     plen = len;
 
@@ -817,16 +918,17 @@ gst_rtp_buffer_get_payload_buffer (GstBuffer * buffer)
 guint
 gst_rtp_buffer_get_payload_len (GstBuffer * buffer)
 {
-  guint len;
+  guint len, size;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), 0);
   g_return_val_if_fail (GST_BUFFER_DATA (buffer) != NULL, 0);
 
-  len = GST_BUFFER_SIZE (buffer)
-      - GST_RTP_HEADER_LEN - GST_RTP_HEADER_CSRC_SIZE (buffer);
+  size = GST_BUFFER_SIZE (buffer);
+
+  len = size - gst_rtp_buffer_get_header_len (buffer);
 
   if (GST_RTP_HEADER_PADDING (buffer))
-    len -= ((guint8 *) GST_BUFFER_DATA (buffer))[GST_BUFFER_SIZE (buffer) - 1];
+    len -= GST_BUFFER_DATA (buffer)[size - 1];
 
   return len;
 }
@@ -846,8 +948,7 @@ gst_rtp_buffer_get_payload (GstBuffer * buffer)
   g_return_val_if_fail (GST_IS_BUFFER (buffer), NULL);
   g_return_val_if_fail (GST_BUFFER_DATA (buffer) != NULL, NULL);
 
-  return GST_BUFFER_DATA (buffer) + GST_RTP_HEADER_LEN
-      + GST_RTP_HEADER_CSRC_SIZE (buffer);
+  return GST_BUFFER_DATA (buffer) + gst_rtp_buffer_get_header_len (buffer);
 }
 
 /**
@@ -864,40 +965,88 @@ gst_rtp_buffer_get_payload (GstBuffer * buffer)
 guint32
 gst_rtp_buffer_default_clock_rate (guint8 payload_type)
 {
-  switch (payload_type) {
-    case GST_RTP_PAYLOAD_PCMU:
-    case GST_RTP_PAYLOAD_GSM:
-    case GST_RTP_PAYLOAD_G723:
-    case GST_RTP_PAYLOAD_DVI4_8000:
-    case GST_RTP_PAYLOAD_LPC:
-    case GST_RTP_PAYLOAD_PCMA:
-    case GST_RTP_PAYLOAD_G722:
-    case GST_RTP_PAYLOAD_G729:
-    case GST_RTP_PAYLOAD_QCELP:
-    case GST_RTP_PAYLOAD_CN:
-    case GST_RTP_PAYLOAD_G728:
-      return 8000;
-    case GST_RTP_PAYLOAD_DVI4_11025:
-      return 11025;
-    case GST_RTP_PAYLOAD_DVI4_16000:
-      return 16000;
-    case GST_RTP_PAYLOAD_DVI4_22050:
-      return 22050;
-    case GST_RTP_PAYLOAD_L16_STEREO:
-    case GST_RTP_PAYLOAD_L16_MONO:
-      return 44100;
-    case GST_RTP_PAYLOAD_MPA:
-    case GST_RTP_PAYLOAD_CELLB:
-    case GST_RTP_PAYLOAD_JPEG:
-    case GST_RTP_PAYLOAD_NV:
-    case GST_RTP_PAYLOAD_H261:
-    case GST_RTP_PAYLOAD_MPV:
-    case GST_RTP_PAYLOAD_MP2T:
-    case GST_RTP_PAYLOAD_H263:
-      return 90000;
-    case GST_RTP_PAYLOAD_1016:
-    case GST_RTP_PAYLOAD_G721:
-    default:
-      return -1;
+  const GstRTPPayloadInfo *info;
+  guint32 res;
+
+  info = gst_rtp_payload_info_for_pt (payload_type);
+  if (!info)
+    return -1;
+
+  res = info->clock_rate;
+  /* 0 means unknown so we have to return -1 from this function */
+  if (res == 0)
+    res = -1;
+
+  return res;
+}
+
+/**
+ * gst_rtp_buffer_compare_seqnum:
+ * @seqnum1: a sequence number
+ * @seqnum2: a sequence number
+ *
+ * Compare two sequence numbers, taking care of wraparounds.
+ *
+ * Returns: -1 if @seqnum1 is before @seqnum2, 0 if they are equal or 1 if
+ * @seqnum1 is bigger than @segnum2.
+ *
+ * Since: 0.10.15
+ */
+gint
+gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
+{
+  /* check if diff more than half of the 16bit range */
+  if (abs (seqnum2 - seqnum1) > (1 << 15)) {
+    /* one of a/b has wrapped */
+    return seqnum1 - seqnum2;
+  } else {
+    return seqnum2 - seqnum1;
   }
 }
+
+/**
+ * gst_rtp_buffer_ext_timestamp:
+ * @exttimestamp: a previous extended timestamp
+ * @timestamp: a new timestamp
+ *
+ * Update the @exttimestamp field with @timestamp. For the first call of the
+ * method, @exttimestamp should point to a location with a value of -1.
+ *
+ * This function makes sure that the returned value is a constantly increasing
+ * value even in the case where there is a timestamp wraparound.
+ *
+ * Returns: The extended timestamp of @timestamp.
+ *
+ * Since: 0.10.15
+ */
+guint64
+gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
+{
+  guint64 result, diff, ext;
+
+  g_return_val_if_fail (exttimestamp != NULL, -1);
+
+  ext = *exttimestamp;
+
+  if (ext == -1) {
+    result = timestamp;
+  } else {
+    /* pick wraparound counter from previous timestamp and add to new timestamp */
+    result = timestamp + (ext & ~(G_GINT64_CONSTANT (0xffffffff)));
+
+    /* check for timestamp wraparound */
+    if (result < ext)
+      diff = ext - result;
+    else
+      diff = result - ext;
+
+    if (diff > G_MAXINT32) {
+      /* timestamp went backwards more than allowed, we wrap around and get
+       * updated extended timestamp. */
+      result += (G_GINT64_CONSTANT (1) << 32);
+    }
+  }
+  *exttimestamp = result;
+
+  return result;
+}
