@@ -66,22 +66,12 @@
 #endif
 
 #include <glib.h>               /* for G_OS_WIN32 */
-#include <gst/gstinfo.h>        /* For GST_STR_NULL */
 
 #ifdef G_OS_WIN32
-/* ws2_32.dll has getaddrinfo and freeaddrinfo on Windows XP and later.
- * minwg32 headers check WINVER before allowing the use of these */
-#ifndef WINVER
-#define WINVER 0x0501
-#endif
-#ifdef _MSC_VER
-#include <Winsock2.h>
-#endif
 #include <ws2tcpip.h>
 #else
 #include <sys/socket.h>
 #include <netdb.h>
-#include <netinet/in.h>
 #endif
 
 #include "gstsdpmessage.h"
@@ -322,56 +312,33 @@ gst_sdp_message_free (GstSDPMessage * msg)
   return GST_SDP_OK;
 }
 
-/**
- * gst_sdp_address_is_multicast:
- * @nettype: a network type
- * @addrtype: an address type
- * @addr: an address
- *
- * Check if the given @addr is a multicast address.
- *
- * Returns: TRUE when @addr is multicast.
- *
- * Since: 0.10.32
- */
-gboolean
-gst_sdp_address_is_multicast (const gchar * nettype, const gchar * addrtype,
-    const gchar * addr)
+static gboolean
+is_multicast_address (const gchar * host_name, guint * family)
 {
-  struct addrinfo hints;
+  struct addrinfo hints = {
+    .ai_socktype = SOCK_DGRAM
+  };
   struct addrinfo *ai;
   struct addrinfo *res;
   gboolean ret = FALSE;
+  int err;
 
-  g_return_val_if_fail (addr, FALSE);
+  g_return_val_if_fail (host_name, FALSE);
 
-  /* we only support IN */
-  if (nettype && strcmp (nettype, "IN") != 0)
-    return FALSE;
-
-  memset (&hints, 0, sizeof (hints));
-  hints.ai_socktype = SOCK_DGRAM;
-
-  /* set the address type as a hint */
-  if (addrtype) {
-    if (!strcmp (addrtype, "IP4"))
-      hints.ai_family = AF_INET;
-    else if (!strcmp (addrtype, "IP6"))
-      hints.ai_family = AF_INET6;
-  }
-
-  if (getaddrinfo (addr, NULL, &hints, &res) < 0)
+  if ((err = getaddrinfo (host_name, NULL, &hints, &res)) < 0)
     return FALSE;
 
   for (ai = res; !ret && ai; ai = ai->ai_next) {
     if (ai->ai_family == AF_INET)
       ret =
-          IN_MULTICAST (ntohl (((struct sockaddr_in *) ai->ai_addr)->
-              sin_addr.s_addr));
+          IN_MULTICAST (ntohl (((struct sockaddr_in *) ai->ai_addr)->sin_addr.
+              s_addr));
     else
       ret =
-          IN6_IS_ADDR_MULTICAST (&((struct sockaddr_in6 *) ai->
-              ai_addr)->sin6_addr);
+          IN6_IS_ADDR_MULTICAST (&((struct sockaddr_in6 *) ai->ai_addr)->
+          sin6_addr);
+    if (ret && family)
+      *family = ai->ai_family;
   }
 
   freeaddrinfo (res);
@@ -431,12 +398,12 @@ gst_sdp_message_as_text (const GstSDPMessage * msg)
 
   if (msg->connection.nettype && msg->connection.addrtype &&
       msg->connection.address) {
+    guint family;
+
     g_string_append_printf (lines, "c=%s %s %s", msg->connection.nettype,
         msg->connection.addrtype, msg->connection.address);
-    if (gst_sdp_address_is_multicast (msg->connection.nettype,
-            msg->connection.addrtype, msg->connection.address)) {
-      /* only add ttl for IP4 */
-      if (strcmp (msg->connection.addrtype, "IP4") == 0)
+    if (is_multicast_address (msg->connection.address, &family)) {
+      if (family == AF_INET)
         g_string_append_printf (lines, "/%u", msg->connection.ttl);
       if (msg->connection.addr_number > 1)
         g_string_append_printf (lines, "/%u", msg->connection.addr_number);
@@ -509,183 +476,6 @@ gst_sdp_message_as_text (const GstSDPMessage * msg)
   return g_string_free (lines, FALSE);
 }
 
-static int
-hex_to_int (gchar c)
-{
-  return c >= '0' && c <= '9' ? c - '0'
-      : c >= 'A' && c <= 'F' ? c - 'A' + 10
-      : c >= 'a' && c <= 'f' ? c - 'a' + 10 : 0;
-}
-
-/**
- * gst_sdp_message_parse_uri:
- * @uri: the start of the uri
- * @msg: the result #GstSDPMessage
- *
- * Parse the null-terminated @uri and store the result in @msg.
- *
- * The uri should be of the form:
- *
- *  scheme://[address[:ttl=ttl][:noa=noa]]/[sessionname]
- *               [#type=value *[&type=value]]
- *
- *  where value is url encoded. This looslely resembles
- *  http://tools.ietf.org/html/draft-fujikawa-sdp-url-01
- *
- * Returns: #GST_SDP_OK on success.
- *
- * Since: 0.10.31
- */
-GstSDPResult
-gst_sdp_message_parse_uri (const gchar * uri, GstSDPMessage * msg)
-{
-  GstSDPResult res;
-  gchar *message;
-  const gchar *colon, *slash, *hash, *p;
-  GString *lines;
-
-  g_return_val_if_fail (uri != NULL, GST_SDP_EINVAL);
-  g_return_val_if_fail (msg != NULL, GST_SDP_EINVAL);
-
-  colon = strstr (uri, "://");
-  if (!colon)
-    goto no_colon;
-
-  /* FIXME connection info goes here */
-
-  slash = strstr (colon + 3, "/");
-  if (!slash)
-    goto no_slash;
-
-  /* FIXME session name goes here */
-
-  hash = strstr (slash + 1, "#");
-  if (!hash)
-    goto no_hash;
-
-  lines = g_string_new ("");
-
-  /* unescape */
-  for (p = hash + 1; *p; p++) {
-    if (*p == '&')
-      g_string_append_printf (lines, "\r\n");
-    else if (*p == '+')
-      g_string_append_c (lines, ' ');
-    else if (*p == '%') {
-      gchar a, b;
-
-      if ((a = p[1])) {
-        if ((b = p[2])) {
-          g_string_append_c (lines, (hex_to_int (a) << 4) | hex_to_int (b));
-          p += 2;
-        }
-      } else {
-        p++;
-      }
-    } else
-      g_string_append_c (lines, *p);
-  }
-
-  message = g_string_free (lines, FALSE);
-  res =
-      gst_sdp_message_parse_buffer ((const guint8 *) message, strlen (message),
-      msg);
-  g_free (message);
-
-  return res;
-
-  /* ERRORS */
-no_colon:
-  {
-    return GST_SDP_EINVAL;
-  }
-no_slash:
-  {
-    return GST_SDP_EINVAL;
-  }
-no_hash:
-  {
-    return GST_SDP_EINVAL;
-  }
-}
-
-static const guchar acceptable[96] = {
-  /* X0   X1    X2    X3    X4    X5    X6    X7    X8    X9    XA    XB    XC    XD    XE    XF */
-  0x00, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00,       /* 2X  !"#$%&'()*+,-./   */
-  0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,       /* 3X 0123456789:;<=>?   */
-  0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,       /* 4X @ABCDEFGHIJKLMNO   */
-  0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01,       /* 5X PQRSTUVWXYZ[\]^_   */
-  0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,       /* 6X `abcdefghijklmno   */
-  0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00        /* 7X pqrstuvwxyz{|}~DEL */
-};
-
-static const gchar hex[16] = "0123456789ABCDEF";
-
-#define ACCEPTABLE_CHAR(a) (((guchar)(a))>=32 && ((guchar)(a))<128 && acceptable[(((guchar)a))-32])
-
-/**
- * gst_sdp_message_as_uri:
- * @scheme: the uri scheme
- * @msg: the #GstSDPMessage
- *
- * Creates a uri from @msg with the given @scheme. The uri has the format:
- *
- *  @scheme:///[#type=value *[&type=value]]
- *
- *  Where each value is url encoded.
- *
- * Returns: a uri for @msg.
- *
- * Since: 0.10.31
- */
-gchar *
-gst_sdp_message_as_uri (const gchar * scheme, const GstSDPMessage * msg)
-{
-  gchar *serialized, *p;
-  gchar *res;
-  GString *lines;
-  gboolean first;
-
-  g_return_val_if_fail (scheme != NULL, NULL);
-  g_return_val_if_fail (msg != NULL, NULL);
-
-  p = serialized = gst_sdp_message_as_text (msg);
-
-  lines = g_string_new ("");
-  g_string_append_printf (lines, "%s:///#", scheme);
-
-  /* now escape */
-  first = TRUE;
-  for (p = serialized; *p; p++) {
-    if (first) {
-      g_string_append_printf (lines, "%c=", *p);
-      if (*(p + 1))
-        p++;
-      first = FALSE;
-      continue;
-    }
-    if (*p == '\r')
-      continue;
-    else if (*p == '\n') {
-      if (*(p + 1))
-        g_string_append_c (lines, '&');
-      first = TRUE;
-    } else if (*p == ' ')
-      g_string_append_c (lines, '+');
-    else if (ACCEPTABLE_CHAR (*p))
-      g_string_append_c (lines, *p);
-    else {
-      /* escape */
-      g_string_append_printf (lines, "%%%c%c", hex[*p >> 4], hex[*p & 0xf]);
-    }
-  }
-
-  res = g_string_free (lines, FALSE);
-  g_free (serialized);
-
-  return res;
-}
-
 /**
  * gst_sdp_message_set_version:
  * @msg: a #GstSDPMessage
@@ -1204,7 +994,7 @@ DEFINE_ARRAY_GETTER (media, medias, const GstSDPMedia);
  *
  * Adds @media to the array of medias in @msg. This function takes ownership of
  * the contents of @media so that @media will have to be reinitialized with
- * gst_sdp_media_init() before it can be used again.
+ * gst_media_init() before it can be used again.
  *
  * Returns: a #GstSDPResult.
  */
@@ -1364,12 +1154,12 @@ gst_sdp_media_as_text (const GstSDPMedia * media)
     const GstSDPConnection *conn = gst_sdp_media_get_connection (media, i);
 
     if (conn->nettype && conn->addrtype && conn->address) {
+      guint family;
+
       g_string_append_printf (lines, "c=%s %s %s", conn->nettype,
           conn->addrtype, conn->address);
-      if (gst_sdp_address_is_multicast (conn->nettype, conn->addrtype,
-              conn->address)) {
-        /* only add TTL for IP4 multicast */
-        if (strcmp (conn->addrtype, "IP4") == 0)
+      if (is_multicast_address (conn->address, &family)) {
+        if (family == AF_INET)
           g_string_append_printf (lines, "/%u", conn->ttl);
         if (conn->addr_number > 1)
           g_string_append_printf (lines, "/%u", conn->addr_number);
@@ -1942,40 +1732,17 @@ gst_sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
       gst_sdp_message_add_phone (c->msg, buffer);
       break;
     case 'c':
-    {
-      GstSDPConnection conn;
-      gchar *str2;
-
-      memset (&conn, 0, sizeof (conn));
-
-      str2 = p;
-      while ((str2 = strchr (str2, '/')))
-        *str2++ = ' ';
-      READ_STRING (conn.nettype);
-      READ_STRING (conn.addrtype);
-      READ_STRING (conn.address);
-      /* only read TTL for IP4 */
-      if (strcmp (conn.addrtype, "IP4") == 0)
-        READ_UINT (conn.ttl);
-      READ_UINT (conn.addr_number);
-
-      if (c->state == SDP_SESSION) {
-        gst_sdp_message_set_connection (c->msg, conn.nettype, conn.addrtype,
-            conn.address, conn.ttl, conn.addr_number);
-      } else {
-        gst_sdp_media_add_connection (c->media, conn.nettype, conn.addrtype,
-            conn.address, conn.ttl, conn.addr_number);
-      }
-      gst_sdp_connection_init (&conn);
+      READ_STRING (c->msg->connection.nettype);
+      READ_STRING (c->msg->connection.addrtype);
+      READ_STRING (c->msg->connection.address);
+      READ_UINT (c->msg->connection.ttl);
+      READ_UINT (c->msg->connection.addr_number);
       break;
-    }
     case 'b':
     {
       gchar str2[MAX_LINE_LEN];
 
       read_string_del (str, sizeof (str), ':', &p);
-      if (*p != '\0')
-        p++;
       read_string (str2, sizeof (str2), &p);
       if (c->state == SDP_SESSION)
         gst_sdp_message_add_bandwidth (c->msg, str, atoi (str2));
@@ -2098,10 +1865,10 @@ gst_sdp_message_parse_buffer (const guint8 * data, guint size,
 static void
 print_media (GstSDPMedia * media)
 {
-  g_print ("   media:       '%s'\n", GST_STR_NULL (media->media));
+  g_print ("   media:       '%s'\n", media->media);
   g_print ("   port:        '%u'\n", media->port);
   g_print ("   num_ports:   '%u'\n", media->num_ports);
-  g_print ("   proto:       '%s'\n", GST_STR_NULL (media->proto));
+  g_print ("   proto:       '%s'\n", media->proto);
   if (media->fmts->len > 0) {
     guint i;
 
@@ -2110,37 +1877,10 @@ print_media (GstSDPMedia * media)
       g_print ("    format  '%s'\n", g_array_index (media->fmts, gchar *, i));
     }
   }
-  g_print ("   information: '%s'\n", GST_STR_NULL (media->information));
-  if (media->connections->len > 0) {
-    guint i;
-
-    g_print ("   connections:\n");
-    for (i = 0; i < media->connections->len; i++) {
-      GstSDPConnection *conn =
-          &g_array_index (media->connections, GstSDPConnection, i);
-
-      g_print ("    nettype:      '%s'\n", GST_STR_NULL (conn->nettype));
-      g_print ("    addrtype:     '%s'\n", GST_STR_NULL (conn->addrtype));
-      g_print ("    address:      '%s'\n", GST_STR_NULL (conn->address));
-      g_print ("    ttl:          '%u'\n", conn->ttl);
-      g_print ("    addr_number:  '%u'\n", conn->addr_number);
-    }
-  }
-  if (media->bandwidths->len > 0) {
-    guint i;
-
-    g_print ("   bandwidths:\n");
-    for (i = 0; i < media->bandwidths->len; i++) {
-      GstSDPBandwidth *bw =
-          &g_array_index (media->bandwidths, GstSDPBandwidth, i);
-
-      g_print ("    type:         '%s'\n", GST_STR_NULL (bw->bwtype));
-      g_print ("    bandwidth:    '%u'\n", bw->bandwidth);
-    }
-  }
+  g_print ("   information: '%s'\n", media->information);
   g_print ("   key:\n");
-  g_print ("    type:       '%s'\n", GST_STR_NULL (media->key.type));
-  g_print ("    data:       '%s'\n", GST_STR_NULL (media->key.data));
+  g_print ("    type:       '%s'\n", media->key.type);
+  g_print ("    data:       '%s'\n", media->key.data);
   if (media->attributes->len > 0) {
     guint i;
 
@@ -2168,17 +1908,17 @@ gst_sdp_message_dump (const GstSDPMessage * msg)
   g_return_val_if_fail (msg != NULL, GST_SDP_EINVAL);
 
   g_print ("sdp packet %p:\n", msg);
-  g_print (" version:       '%s'\n", GST_STR_NULL (msg->version));
+  g_print (" version:       '%s'\n", msg->version);
   g_print (" origin:\n");
-  g_print ("  username:     '%s'\n", GST_STR_NULL (msg->origin.username));
-  g_print ("  sess_id:      '%s'\n", GST_STR_NULL (msg->origin.sess_id));
-  g_print ("  sess_version: '%s'\n", GST_STR_NULL (msg->origin.sess_version));
-  g_print ("  nettype:      '%s'\n", GST_STR_NULL (msg->origin.nettype));
-  g_print ("  addrtype:     '%s'\n", GST_STR_NULL (msg->origin.addrtype));
-  g_print ("  addr:         '%s'\n", GST_STR_NULL (msg->origin.addr));
-  g_print (" session_name:  '%s'\n", GST_STR_NULL (msg->session_name));
-  g_print (" information:   '%s'\n", GST_STR_NULL (msg->information));
-  g_print (" uri:           '%s'\n", GST_STR_NULL (msg->uri));
+  g_print ("  username:     '%s'\n", msg->origin.username);
+  g_print ("  sess_id:      '%s'\n", msg->origin.sess_id);
+  g_print ("  sess_version: '%s'\n", msg->origin.sess_version);
+  g_print ("  nettype:      '%s'\n", msg->origin.nettype);
+  g_print ("  addrtype:     '%s'\n", msg->origin.addrtype);
+  g_print ("  addr:         '%s'\n", msg->origin.addr);
+  g_print (" session_name:  '%s'\n", msg->session_name);
+  g_print (" information:   '%s'\n", msg->information);
+  g_print (" uri:           '%s'\n", msg->uri);
 
   if (msg->emails->len > 0) {
     guint i;
@@ -2197,26 +1937,14 @@ gst_sdp_message_dump (const GstSDPMessage * msg)
     }
   }
   g_print (" connection:\n");
-  g_print ("  nettype:      '%s'\n", GST_STR_NULL (msg->connection.nettype));
-  g_print ("  addrtype:     '%s'\n", GST_STR_NULL (msg->connection.addrtype));
-  g_print ("  address:      '%s'\n", GST_STR_NULL (msg->connection.address));
+  g_print ("  nettype:      '%s'\n", msg->connection.nettype);
+  g_print ("  addrtype:     '%s'\n", msg->connection.addrtype);
+  g_print ("  address:      '%s'\n", msg->connection.address);
   g_print ("  ttl:          '%u'\n", msg->connection.ttl);
   g_print ("  addr_number:  '%u'\n", msg->connection.addr_number);
-  if (msg->bandwidths->len > 0) {
-    guint i;
-
-    g_print (" bandwidths:\n");
-    for (i = 0; i < msg->bandwidths->len; i++) {
-      GstSDPBandwidth *bw =
-          &g_array_index (msg->bandwidths, GstSDPBandwidth, i);
-
-      g_print ("  type:         '%s'\n", GST_STR_NULL (bw->bwtype));
-      g_print ("  bandwidth:    '%u'\n", bw->bandwidth);
-    }
-  }
   g_print (" key:\n");
-  g_print ("  type:         '%s'\n", GST_STR_NULL (msg->key.type));
-  g_print ("  data:         '%s'\n", GST_STR_NULL (msg->key.data));
+  g_print ("  type:         '%s'\n", msg->key.type);
+  g_print ("  data:         '%s'\n", msg->key.data);
   if (msg->attributes->len > 0) {
     guint i;
 
