@@ -197,7 +197,9 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM, "TALB"},
   {GST_TAG_TRACK_NUMBER, "TRCK"},
   {GST_TAG_ARTIST, "TPE1"},
+  {GST_TAG_COMPOSER, "TCOM"},
   {GST_TAG_COPYRIGHT, "TCOP"},
+  {GST_TAG_COPYRIGHT_URI, "WCOP"},
   {GST_TAG_GENRE, "TCON"},
   {GST_TAG_DATE, "TDRC"},
   {GST_TAG_COMMENT, "COMM"},
@@ -207,7 +209,9 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_IMAGE, "APIC"},
   {GST_TAG_ENCODER, "TSSE"},
   {GST_TAG_BEATS_PER_MINUTE, "TBPM"},
-  {GST_TAG_MUSICBRAINZ_SORTNAME, "TSOP"},
+  {GST_TAG_ARTIST_SORTNAME, "TSOP"},
+  {GST_TAG_ALBUM_SORTNAME, "TSOA"},
+  {GST_TAG_TITLE_SORTNAME, "TSOT"},
   {NULL, NULL}
 };
 
