@@ -71,6 +71,7 @@ struct _GstDecodeBin
   gint numpads;
   gint numwaiting;
 
+  gboolean have_type;
   guint have_type_id;           /* signal id for the typefind element */
 
   gboolean shutting_down;       /* stop pluggin if we're shutting down */
@@ -583,6 +584,7 @@ add_fakesink (GstDecodeBin * decode_bin)
   /* hacky, remove sink flag, we don't want our decodebin to become a sink
    * just because we add a fakesink element to make us ASYNC */
   GST_OBJECT_FLAG_UNSET (decode_bin->fakesink, GST_ELEMENT_IS_SINK);
+
   /* takes ownership */
   if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->fakesink)) {
     g_warning ("Could not add fakesink element, decodebin will not work");
@@ -749,7 +751,7 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
     GList *to_try;
 
     /* if the caps has many types, we need to delay */
-    if (gst_caps_get_size (caps) != 1)
+    if (!gst_caps_is_fixed (caps))
       goto many_types;
 
     /* continue plugging, first find all compatible elements */
@@ -778,6 +780,19 @@ unknown_type:
 
     gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
         gst_missing_decoder_message_new (GST_ELEMENT_CAST (decode_bin), caps));
+
+    if (element == decode_bin->typefind) {
+      gchar *desc;
+
+      desc = gst_pb_utils_get_decoder_description (caps);
+      GST_ELEMENT_ERROR (decode_bin, STREAM, CODEC_NOT_FOUND,
+          (_("A %s plugin is required to play this stream, but not installed."),
+              desc),
+          ("No decoder to handle media type '%s'",
+              gst_structure_get_name (gst_caps_get_structure (caps, 0))));
+      g_free (desc);
+    }
+
     return;
   }
 dont_know_yet:
@@ -1259,11 +1274,13 @@ new_pad (GstElement * element, GstPad * pad, GstDynamic * dynamic)
 
 shutting_down1:
   {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
     GST_OBJECT_UNLOCK (decode_bin);
     return;
   }
 shutting_down2:
   {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
     GST_STATE_UNLOCK (decode_bin);
     return;
   }
@@ -1535,14 +1552,26 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
   gboolean dynamic;
   GstPad *pad;
 
-  GST_STATE_LOCK (decode_bin);
-  if (decode_bin->shutting_down)
-    goto shutting_down;
-
   GST_DEBUG_OBJECT (decode_bin, "typefind found caps %" GST_PTR_FORMAT, caps);
 
+  GST_OBJECT_LOCK (decode_bin);
+  if (decode_bin->shutting_down)
+    goto shutting_down;
+  GST_OBJECT_UNLOCK (decode_bin);
+
+  GST_STATE_LOCK (decode_bin);
+  if (decode_bin->shutting_down)
+    goto exit;
+
+  /* don't need the typefind anymore if we already found a type, we're not going
+   * to be able to do anything with it anyway except for generating errors */
+  if (decode_bin->have_type)
+    goto exit;
+
+  decode_bin->have_type = TRUE;
+
   /* special-case text/plain: we only want to accept it as a raw type if it
-   * comes from a subtitel parser element or a demuxer, but not if it is the
+   * comes from a subtitle parser element or a demuxer, but not if it is the
    * type of the entire stream, in which case we just want to error out */
   if (typefind == decode_bin->typefind &&
       gst_structure_has_name (gst_caps_get_structure (caps, 0), "text/plain")) {
@@ -1551,7 +1580,7 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
     GST_ELEMENT_ERROR (decode_bin, STREAM, WRONG_TYPE,
         (_("This appears to be a text file")),
         ("decodebin cannot decode plain text files"));
-    goto shutting_down;
+    goto exit;
   }
 
   /* autoplug the new pad with the caps that the signal gave us. */
@@ -1570,9 +1599,16 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
     GST_DEBUG_OBJECT (decode_bin, "we have more dynamic elements");
   }
 
-shutting_down:
+exit:
   GST_STATE_UNLOCK (decode_bin);
   return;
+
+shutting_down:
+  {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
+    GST_OBJECT_UNLOCK (decode_bin);
+    return;
+  }
 }
 
 static void
@@ -1703,6 +1739,7 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_OBJECT_LOCK (decode_bin);
       decode_bin->shutting_down = FALSE;
+      decode_bin->have_type = FALSE;
       GST_OBJECT_UNLOCK (decode_bin);
 
       if (!add_fakesink (decode_bin))
