@@ -82,6 +82,10 @@ struct _GstDecodeBin
 {
   GstBin bin;                   /* we extend GstBin */
 
+  /* properties */
+  GstCaps *caps;                /* caps on which to stop decoding */
+  gchar *encoding;              /* encoding of subtitles */
+
   GstElement *typefind;         /* this holds the typefind object */
   GstElement *fakesink;
 
@@ -92,9 +96,11 @@ struct _GstDecodeBin
   GList *oldgroups;             /* List of no-longer-used GstDecodeGroups. 
                                  * Should be freed in dispose */
   gint nbpads;                  /* unique identifier for source pads */
-  GstCaps *caps;                /* caps on which to stop decoding */
 
   GList *factories;             /* factories we can use for selecting elements */
+
+  gboolean have_type;           /* if we received the have_type signal */
+  guint have_type_id;           /* signal id for have-type from typefind */
 };
 
 struct _GstDecodeBinClass
@@ -107,11 +113,16 @@ struct _GstDecodeBinClass
   void (*removed_decoded_pad) (GstElement * element, GstPad * pad);
   /* signal fired when we found a pad that we cannot decode */
   void (*unknown_type) (GstElement * element, GstPad * pad, GstCaps * caps);
+
   /* signal fired to know if we continue trying to decode the given caps */
-    gboolean (*autoplug_continue) (GstElement * element, GstCaps * caps);
-  /* signal fired to reorder the proposed list of factories */
-    gboolean (*autoplug_sort) (GstElement * element, GstCaps * caps,
-      GList ** list);
+    gboolean (*autoplug_continue) (GstElement * element, GstPad * pad,
+      GstCaps * caps);
+  /* signal fired to get a list of factories to try to autoplug */
+  GValueArray *(*autoplug_factories) (GstElement * element, GstPad * pad,
+      GstCaps * caps);
+  /* signal fired to select from the proposed list of factories */
+    gint (*autoplug_select) (GstElement * element, GstPad * pad, GstCaps * caps,
+      GValueArray * factories);
 };
 
 /* signals */
@@ -121,7 +132,8 @@ enum
   SIGNAL_REMOVED_DECODED_PAD,
   SIGNAL_UNKNOWN_TYPE,
   SIGNAL_AUTOPLUG_CONTINUE,
-  SIGNAL_AUTOPLUG_SORT,
+  SIGNAL_AUTOPLUG_FACTORIES,
+  SIGNAL_AUTOPLUG_SELECT,
   LAST_SIGNAL
 };
 
@@ -130,6 +142,7 @@ enum
 {
   PROP_0,
   PROP_CAPS,
+  PROP_SUBTITLE_ENCODING
 };
 
 static GstBinClass *parent_class;
@@ -149,9 +162,12 @@ static void type_found (GstElement * typefind, guint probability,
     GstCaps * caps, GstDecodeBin * decode_bin);
 
 static gboolean gst_decode_bin_autoplug_continue (GstElement * element,
-    GstCaps * caps);
-static gboolean gst_decode_bin_autoplug_sort (GstElement * element,
-    GstCaps * caps, GList ** list);
+    GstPad * pad, GstCaps * caps);
+static GValueArray *gst_decode_bin_autoplug_factories (GstElement *
+    element, GstPad * pad, GstCaps * caps);
+static gint gst_decode_bin_autoplug_select (GstElement * element,
+    GstPad * pad, GstCaps * caps, GValueArray * list);
+
 static void gst_decode_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_decode_bin_get_property (GObject * object, guint prop_id,
@@ -206,6 +222,8 @@ struct _GstDecodeGroup
 
   GList *endpads;               /* List of GstDecodePad of source pads to be exposed */
   GList *ghosts;                /* List of GstGhostPad for the endpads */
+
+  GList *reqpads;               /* List of RequestPads */
 };
 
 #define GROUP_MUTEX_LOCK(group) G_STMT_START {				\
@@ -341,7 +359,6 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    * This signal gets emitted as soon as a new pad of the same type as one of
    * the valid 'raw' types is added.
    */
-
   gst_decode_bin_signals[SIGNAL_NEW_DECODED_PAD] =
       g_signal_new ("new-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
@@ -355,7 +372,6 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    *
    * This signal is emitted when a 'final' caps pad has been removed.
    */
-
   gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD] =
       g_signal_new ("removed-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
@@ -370,7 +386,6 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    * This signal is emitted when a pad for which there is no further possible
    * decoding is added to the decodebin.
    */
-
   gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE] =
       g_signal_new ("unknown-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, unknown_type),
@@ -379,6 +394,7 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
 
   /**
    * GstDecodeBin2::autoplug-continue:
+   * @pad: The #GstPad.
    * @caps: The #GstCaps found.
    *
    * This signal is emitted whenever decodebin2 finds a new stream. It is
@@ -389,42 +405,68 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    * final and the pad will be exposed as such (see 'new-decoded-pad'
    * signal).
    */
-
   gst_decode_bin_signals[SIGNAL_AUTOPLUG_CONTINUE] =
       g_signal_new ("autoplug-continue", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_continue),
-      _gst_boolean_accumulator, NULL, gst_play_marshal_BOOLEAN__OBJECT,
-      G_TYPE_BOOLEAN, 1, GST_TYPE_CAPS);
+      _gst_boolean_accumulator, NULL, gst_play_marshal_BOOLEAN__OBJECT_OBJECT,
+      G_TYPE_BOOLEAN, 2, GST_TYPE_PAD, GST_TYPE_CAPS);
 
   /**
-   * GstDecodeBin2::autoplug-sort:
+   * GstDecodeBin2::autoplug-factories:
+   * @pad: The #GstPad.
+   * @caps: The #GstCaps found.
+   *
+   * This function is emited when an array of possible factories for @caps on
+   * @pad is needed. Decodebin2 will by default return 
+   *
+   * Returns: a #GValueArray* with a list of factories to try. The factories are
+   * by default tried in the returned order or based on the index returned by
+   * "autoplug-select".
+   */
+  gst_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES] =
+      g_signal_new ("autoplug-factories", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass,
+          autoplug_factories), NULL, NULL,
+      gst_play_marshal_BOXED__OBJECT_OBJECT, G_TYPE_VALUE_ARRAY, 2,
+      GST_TYPE_PAD, GST_TYPE_CAPS);
+
+  /**
+   * GstDecodeBin2::autoplug-select:
+   * @pad: The #GstPad.
    * @caps: The #GstCaps.
-   * @factories: A #GList of possible #GstElementFactory to use.
+   * @factories: A #GValueArray of possible #GstElementFactory to use, sorted by
+   * rank (higher ranks come first).
    *
    * This signal is emitted once decodebin2 has found all the possible
    * #GstElementFactory that can be used to handle the given @caps.
    *
-   * UNSTABLE API. Will change soon.
-   *
-   * Returns: #TRUE if you wish decodebin2 to start trying to decode
-   * the given @caps with the list of factories. #FALSE if you do not want
-   * these #GstCaps, if so the pad will be exposed as unknown (see
-   * 'unknown-type' signal).
+   * Returns: A #gint indicating what factory index from the @factories array
+   * that you wish decodebin2 to use for trying to decode the given @caps.
+   * -1 to stop selection of a factory. The default handler always
+   * returns the first possible factory.
    */
-
-  gst_decode_bin_signals[SIGNAL_AUTOPLUG_SORT] =
-      g_signal_new ("autoplug-sort", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_sort),
-      _gst_boolean_accumulator, NULL, gst_play_marshal_BOOLEAN__OBJECT_POINTER,
-      G_TYPE_BOOLEAN, 2, GST_TYPE_CAPS, G_TYPE_POINTER);
+  gst_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT] =
+      g_signal_new ("autoplug-select", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_select),
+      NULL, NULL, gst_play_marshal_INT__OBJECT_OBJECT_BOXED,
+      G_TYPE_INT, 3, GST_TYPE_PAD, GST_TYPE_CAPS, G_TYPE_VALUE_ARRAY);
 
   g_object_class_install_property (gobject_klass, PROP_CAPS,
       g_param_spec_boxed ("caps", "Caps", "The caps on which to stop decoding.",
           GST_TYPE_CAPS, G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_klass, PROP_SUBTITLE_ENCODING,
+      g_param_spec_string ("subtitle-encoding", "subtitle encoding",
+          "Encoding to assume if input subtitles are not in UTF-8 encoding. "
+          "If not set, the GST_SUBTITLE_ENCODING environment variable will "
+          "be checked for an encoding to use. If that is not set either, "
+          "ISO-8859-15 will be assumed.", NULL, G_PARAM_READWRITE));
+
   klass->autoplug_continue =
       GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_continue);
-  klass->autoplug_sort = GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_sort);
+  klass->autoplug_factories =
+      GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_factories);
+  klass->autoplug_select = GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_select);
 
   gst_element_class_add_pad_template (gstelement_klass,
       gst_static_pad_template_get (&decoder_bin_sink_template));
@@ -451,10 +493,11 @@ gst_decode_bin_factory_filter (GstPluginFeature * feature,
     return FALSE;
 
   klass = gst_element_factory_get_klass (GST_ELEMENT_FACTORY (feature));
-  /* only demuxers, decoders and parsers can play */
+  /* only demuxers, decoders, depayloaders and parsers can play */
   if (strstr (klass, "Demux") == NULL &&
-      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL &&
-      strstr (klass, "Depayloader") == NULL) {
+      strstr (klass, "Decoder") == NULL &&
+      strstr (klass, "Depayloader") == NULL &&
+      strstr (klass, "Parse") == NULL) {
     return FALSE;
   }
 
@@ -538,7 +581,8 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
 
     /* connect a signal to find out when the typefind element found
      * a type */
-    g_signal_connect (G_OBJECT (decode_bin->typefind), "have_type",
+    decode_bin->have_type_id =
+        g_signal_connect (G_OBJECT (decode_bin->typefind), "have-type",
         G_CALLBACK (type_found), decode_bin);
   }
 
@@ -592,6 +636,10 @@ gst_decode_bin_dispose (GObject * object)
   if (decode_bin->caps)
     gst_caps_unref (decode_bin->caps);
   decode_bin->caps = NULL;
+
+  g_free (decode_bin->encoding);
+  decode_bin->encoding = NULL;
+
   remove_fakesink (decode_bin);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -612,52 +660,14 @@ gst_decode_bin_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-gst_decode_bin_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstDecodeBin *dbin;
-
-  dbin = GST_DECODE_BIN (object);
-
-  switch (prop_id) {
-    case PROP_CAPS:
-      gst_decode_bin_set_caps (dbin, (GstCaps *) g_value_dup_boxed (value));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_decode_bin_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstDecodeBin *dbin;
-
-  dbin = GST_DECODE_BIN (object);
-  switch (prop_id) {
-    case PROP_CAPS:{
-      g_value_take_boxed (value, gst_decode_bin_get_caps (dbin));
-      break;
-    }
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-
-}
-
 /* _set_caps
  * Changes the caps on which decodebin will stop decoding.
  * Will unref the previously set one. The refcount of the given caps will be
- * taken.
+ * increased.
  * @caps can be NULL.
  *
  * MT-safe
  */
-
 static void
 gst_decode_bin_set_caps (GstDecodeBin * dbin, GstCaps * caps)
 {
@@ -666,6 +676,8 @@ gst_decode_bin_set_caps (GstDecodeBin * dbin, GstCaps * caps)
   DECODE_BIN_LOCK (dbin);
   if (dbin->caps)
     gst_caps_unref (dbin->caps);
+  if (caps)
+    gst_caps_ref (caps);
   dbin->caps = caps;
   DECODE_BIN_UNLOCK (dbin);
 }
@@ -693,24 +705,109 @@ gst_decode_bin_get_caps (GstDecodeBin * dbin)
   return caps;
 }
 
+static void
+gst_decode_bin_set_subs_encoding (GstDecodeBin * dbin, const gchar * encoding)
+{
+  GST_DEBUG_OBJECT (dbin, "Setting new encoding: %s", GST_STR_NULL (encoding));
+
+  DECODE_BIN_LOCK (dbin);
+  g_free (dbin->encoding);
+  dbin->encoding = g_strdup (encoding);
+  DECODE_BIN_UNLOCK (dbin);
+}
+
+static gchar *
+gst_decode_bin_get_subs_encoding (GstDecodeBin * dbin)
+{
+  gchar *encoding;
+
+  GST_DEBUG_OBJECT (dbin, "Getting currently set encoding");
+
+  DECODE_BIN_LOCK (dbin);
+  encoding = g_strdup (dbin->encoding);
+  DECODE_BIN_UNLOCK (dbin);
+
+  return encoding;
+}
+
+static void
+gst_decode_bin_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstDecodeBin *dbin;
+
+  dbin = GST_DECODE_BIN (object);
+
+  switch (prop_id) {
+    case PROP_CAPS:
+      gst_decode_bin_set_caps (dbin, g_value_get_boxed (value));
+      break;
+    case PROP_SUBTITLE_ENCODING:
+      gst_decode_bin_set_subs_encoding (dbin, g_value_get_string (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_decode_bin_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstDecodeBin *dbin;
+
+  dbin = GST_DECODE_BIN (object);
+  switch (prop_id) {
+    case PROP_CAPS:
+      g_value_take_boxed (value, gst_decode_bin_get_caps (dbin));
+      break;
+    case PROP_SUBTITLE_ENCODING:
+      g_value_take_string (value, gst_decode_bin_get_subs_encoding (dbin));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+
+static GValueArray *find_compatibles (GstDecodeBin * decode_bin,
+    GstPad * pad, const GstCaps * caps);
+
 /*****
  * Default autoplug signal handlers
  *****/
-
 static gboolean
-gst_decode_bin_autoplug_continue (GstElement * element, GstCaps * caps)
+gst_decode_bin_autoplug_continue (GstElement * element, GstPad * pad,
+    GstCaps * caps)
 {
+  /* by default we always continue */
   return TRUE;
 }
 
-static gboolean
-gst_decode_bin_autoplug_sort (GstElement * element, GstCaps * caps,
-    GList ** list)
+static GValueArray *
+gst_decode_bin_autoplug_factories (GstElement * element, GstPad * pad,
+    GstCaps * caps)
 {
-  return TRUE;
+  GValueArray *result;
+
+  /* return all compatible factories for caps */
+  result = find_compatibles (GST_DECODE_BIN (element), pad, caps);
+
+  return result;
 }
 
+static gint
+gst_decode_bin_autoplug_select (GstElement * element, GstPad * pad,
+    GstCaps * caps, GValueArray * factories)
+{
+  g_return_val_if_fail (factories != NULL, -1);
+  g_return_val_if_fail (factories->n_values > 0, -1);
 
+  /* Return first factory. */
+  return 0;
+}
 
 /********
  * Discovery methods
@@ -718,11 +815,10 @@ gst_decode_bin_autoplug_sort (GstElement * element, GstCaps * caps,
 
 static gboolean are_raw_caps (GstDecodeBin * dbin, GstCaps * caps);
 static gboolean is_demuxer_element (GstElement * srcelement);
-static GList *find_compatibles (GstDecodeBin * decode_bin,
-    const GstCaps * caps);
 
 static gboolean connect_pad (GstDecodeBin * dbin, GstElement * src,
-    GstPad * pad, GList * factories, GstDecodeGroup * group);
+    GstPad * pad, GstCaps * caps, GValueArray * factories,
+    GstDecodeGroup * group);
 static gboolean connect_element (GstDecodeBin * dbin, GstElement * element,
     GstDecodeGroup * group);
 static void expose_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
@@ -742,13 +838,24 @@ static void no_more_pads_cb (GstElement * element, GstDecodeBin * dbin);
 
 static GstDecodeGroup *get_current_group (GstDecodeBin * dbin);
 
+/* called when a new pad is discovered. It will perform some basic actions
+ * before trying to link something to it.
+ *
+ *  - Check the caps, don't do anything when there are no caps or when they have
+ *    no good type.
+ *  - signal AUTOPLUG_CONTINUE to check if we need to continue autoplugging this
+ *    pad.
+ *  - if the caps are non-fixed, setup a handler to continue autoplugging when
+ *    the caps become fixed (connect to notify::caps).
+ *  - get list of factories to autoplug.
+ *  - continue autoplugging to one of the factories.
+ */
 static void
 analyze_new_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
     GstCaps * caps, GstDecodeGroup * group)
 {
   gboolean apcontinue = TRUE;
-  GList *factories = NULL;
-  gboolean apsort = TRUE;
+  GValueArray *factories = NULL;
 
   GST_DEBUG_OBJECT (dbin, "Pad %s:%s caps:%" GST_PTR_FORMAT,
       GST_DEBUG_PAD_NAME (pad), caps);
@@ -759,9 +866,11 @@ analyze_new_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
   if (gst_caps_is_any (caps))
     goto any_caps;
 
-  /* 1. Emit 'autoplug-continue' */
+  /* 1. Emit 'autoplug-continue' the result will tell us if this pads needs
+   * further autoplugging. */
   g_signal_emit (G_OBJECT (dbin),
-      gst_decode_bin_signals[SIGNAL_AUTOPLUG_CONTINUE], 0, caps, &apcontinue);
+      gst_decode_bin_signals[SIGNAL_AUTOPLUG_CONTINUE], 0, pad, caps,
+      &apcontinue);
 
   /* 1.a if autoplug-continue is FALSE or caps is a raw format, goto pad_is_final */
   if ((!apcontinue) || are_raw_caps (dbin, caps))
@@ -772,27 +881,21 @@ analyze_new_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
   if (!gst_caps_is_fixed (caps))
     goto non_fixed;
 
-  /* 1.c else if there's no compatible factory or 'autoplug-sort' returned FALSE, goto pad_not_used */
-  if ((factories = find_compatibles (dbin, caps))) {
-    /* emit autoplug-sort */
-    g_signal_emit (G_OBJECT (dbin),
-        gst_decode_bin_signals[SIGNAL_AUTOPLUG_SORT],
-        0, caps, &factories, &apsort);
-    if (!apsort) {
-      g_list_free (factories);
-      /* User doesn't want that pad */
-      goto pad_not_wanted;
-    }
-  } else {
+  /* 1.c else get the factories and if there's no compatible factory goto
+   * unknown_type */
+  g_signal_emit (G_OBJECT (dbin),
+      gst_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES], 0, pad, caps,
+      &factories);
+  if (factories == NULL) {
     /* no compatible factories */
     goto unknown_type;
   }
 
-  /* 1.d else goto pad_is_valid */
+  /* 1.d else continue autoplugging something from the list. */
   GST_LOG_OBJECT (pad, "Let's continue discovery on this pad");
+  connect_pad (dbin, src, pad, caps, factories, group);
 
-  connect_pad (dbin, src, pad, factories, group);
-  g_list_free (factories);
+  g_value_array_free (factories);
 
   return;
 
@@ -802,13 +905,6 @@ expose_pad:
     expose_pad (dbin, src, pad, group);
     return;
   }
-
-pad_not_wanted:
-  {
-    GST_LOG_OBJECT (pad, "User doesn't want this pad, stopping discovery");
-    return;
-  }
-
 unknown_type:
   {
     GST_LOG_OBJECT (pad, "Unknown type, firing signal");
@@ -819,11 +915,22 @@ unknown_type:
     if (dbin->groups == NULL)
       remove_fakesink (dbin);
 
+    if (src == dbin->typefind) {
+      gchar *desc;
+
+      desc = gst_pb_utils_get_decoder_description (caps);
+      GST_ELEMENT_ERROR (dbin, STREAM, CODEC_NOT_FOUND,
+          (_("A %s plugin is required to play this stream, but not installed."),
+              desc),
+          ("No decoder to handle media type '%s'",
+              gst_structure_get_name (gst_caps_get_structure (caps, 0))));
+      g_free (desc);
+    }
+
     gst_element_post_message (GST_ELEMENT_CAST (dbin),
         gst_missing_decoder_message_new (GST_ELEMENT_CAST (dbin), caps));
     return;
   }
-
 non_fixed:
   {
     GST_DEBUG_OBJECT (pad, "pad has non-fixed caps delay autoplugging");
@@ -860,16 +967,16 @@ setup_caps_delay:
  *
  * Returns TRUE if an element was properly created and linked
  */
-
 static gboolean
 connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
-    GList * factories, GstDecodeGroup * group)
+    GstCaps * caps, GValueArray * factories, GstDecodeGroup * group)
 {
   gboolean res = FALSE;
-  GList *tmp;
   GstPad *mqpad = NULL;
 
   g_return_val_if_fail (factories != NULL, FALSE);
+  g_return_val_if_fail (factories->n_values > 0, FALSE);
+
   GST_DEBUG_OBJECT (dbin, "pad %s:%s , group:%p",
       GST_DEBUG_PAD_NAME (pad), group);
 
@@ -891,11 +998,33 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
   }
 
   /* 2. Try to create an element and link to it */
-  for (tmp = factories; tmp; tmp = g_list_next (tmp)) {
-    GstElementFactory *factory = (GstElementFactory *) tmp->data;
+  while (factories->n_values > 0) {
+    gint factidx;
+    GstElementFactory *factory;
     GstElement *element;
     GstPad *sinkpad;
 
+    /* emit autoplug-select */
+    g_signal_emit (G_OBJECT (dbin),
+        gst_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT],
+        0, pad, caps, factories, &factidx);
+
+    /* when the function returns -1, we leave the loop and stop */
+    if (factidx == -1) {
+      GST_DEBUG_OBJECT (dbin, "autoplug select returned -1");
+      break;
+    }
+
+    GST_DEBUG_OBJECT (dbin, "autoplug select returned %d", factidx);
+
+    factory = g_value_get_object (g_value_array_get_nth (factories, factidx));
+
+    /* factory is != NULL now since the default handler selects the first
+     * factory if all connected user handlers returned NULL. */
+
+    /* Remove selected factory from the list. */
+    g_value_array_remove (factories, factidx);
+
     /* 2.1. Try to create an element */
     if ((element = gst_element_factory_create (factory, NULL)) == NULL) {
       GST_WARNING_OBJECT (dbin, "Could not create an element from %s",
@@ -1136,17 +1265,22 @@ type_found (GstElement * typefind, guint probability,
 {
   GstPad *pad;
 
-  GST_STATE_LOCK (decode_bin);
-
   GST_DEBUG_OBJECT (decode_bin, "typefind found caps %" GST_PTR_FORMAT, caps);
 
-  pad = gst_element_get_pad (typefind, "src");
+  /* we can only deal with one type, we don't yet support dynamically changing
+   * caps from the typefind element */
+  if (decode_bin->have_type)
+    goto exit;
+
+  decode_bin->have_type = TRUE;
+
+  pad = gst_element_get_static_pad (typefind, "src");
 
   analyze_new_pad (decode_bin, typefind, pad, caps, NULL);
 
   gst_object_unref (pad);
 
-  GST_STATE_UNLOCK (decode_bin);
+exit:
   return;
 }
 
@@ -1156,7 +1290,7 @@ pad_added_group_cb (GstElement * element, GstPad * pad, GstDecodeGroup * group)
   GstCaps *caps;
   gboolean expose = FALSE;
 
-  GST_LOG_OBJECT (pad, "pad added, group:%p", group);
+  GST_DEBUG_OBJECT (pad, "pad added, group:%p", group);
 
   caps = gst_pad_get_caps (pad);
   analyze_new_pad (group->dbin, element, pad, caps, group);
@@ -1169,6 +1303,7 @@ pad_added_group_cb (GstElement * element, GstPad * pad, GstDecodeGroup * group)
   if (group->nbdynamic == 0)
     expose = TRUE;
   GROUP_MUTEX_UNLOCK (group);
+
   if (expose) {
     GST_LOG
         ("That was the last dynamic object, now attempting to expose the group");
@@ -1266,14 +1401,14 @@ caps_notify_group_cb (GstPad * pad, GParamSpec * unused, GstDecodeGroup * group)
   gst_object_unref (element);
 }
 
-/* this function runs through the element factories and returns a list
- * of all elements that are able to sink the given caps 
+/* this function runs through the element factories and returns a value array of
+ * all elements that are able to sink the given caps
  */
-static GList *
-find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
+static GValueArray *
+find_compatibles (GstDecodeBin * decode_bin, GstPad * pad, const GstCaps * caps)
 {
   GList *factories;
-  GList *to_try = NULL;
+  GValueArray *to_try = g_value_array_new (0);
 
   /* loop over all the factories */
   for (factories = decode_bin->factories; factories;
@@ -1301,7 +1436,11 @@ find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
         /* check if the intersection is empty */
         if (!gst_caps_is_empty (intersect)) {
           /* non empty intersection, we can use this element */
-          to_try = g_list_prepend (to_try, factory);
+          GValue val = { 0, };
+          g_value_init (&val, G_TYPE_OBJECT);
+          g_value_set_object (&val, factory);
+          g_value_array_append (to_try, &val);
+          g_value_unset (&val);
           gst_caps_unref (intersect);
           break;
         }
@@ -1309,7 +1448,6 @@ find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
       }
     }
   }
-  to_try = g_list_reverse (to_try);
 
   return to_try;
 }
@@ -1455,9 +1593,10 @@ gst_decode_group_new (GstDecodeBin * dbin, gboolean use_queue)
   group->blocked = FALSE;
   group->complete = FALSE;
   group->endpads = NULL;
+  group->reqpads = NULL;
 
   if (mq) {
-    /* we first configure the mulitqueue to buffer an unlimited number of
+    /* we first configure the multiqueue to buffer an unlimited number of
      * buffers up to 5 seconds or, when no timestamps are present, up to 2 MB of
      * memory. When this queue overruns, we assume the group is complete and can
      * be exposed. */
@@ -1542,7 +1681,7 @@ gst_decode_group_control_demuxer_pad (GstDecodeGroup * group, GstPad * pad)
 
   srcpad = NULL;
 
-  if (!(sinkpad = gst_element_get_pad (group->multiqueue, "sink%d"))) {
+  if (!(sinkpad = gst_element_get_request_pad (group->multiqueue, "sink%d"))) {
     GST_ERROR ("Couldn't get sinkpad from multiqueue");
     return NULL;
   }
@@ -1552,6 +1691,8 @@ gst_decode_group_control_demuxer_pad (GstDecodeGroup * group, GstPad * pad)
     goto beach;
   }
 
+  group->reqpads = g_list_append (group->reqpads, sinkpad);
+
   sinkname = gst_pad_get_name (sinkpad);
   nb = sinkname + 4;
   srcname = g_strdup_printf ("src%s", nb);
@@ -1824,13 +1965,24 @@ gst_decode_group_expose (GstDecodeGroup * group)
         gst_decode_bin_signals[SIGNAL_NEW_DECODED_PAD], 0, ghost,
         (next == NULL));
     GST_DEBUG_OBJECT (group->dbin, "emitted new-decoded-pad");
+  }
+
+  /* signal no-more-pads. This allows the application to hook stuff to the
+   * exposed pads */
+  GST_LOG_OBJECT (group->dbin, "signalling no-more-pads");
+  gst_element_no_more_pads (GST_ELEMENT (group->dbin));
+
+  /* 3. Unblock internal pads. The application should have connected stuff now
+   * so that streaming can continue. */
+  for (tmp = group->endpads; tmp; tmp = next) {
+    GstDecodePad *dpad = (GstDecodePad *) tmp->data;
+
+    next = g_list_next (tmp);
 
-    /* 3. Unblock internal  pad */
     GST_DEBUG_OBJECT (dpad->pad, "unblocking");
     gst_pad_set_blocked_async (dpad->pad, FALSE,
         (GstPadBlockCallback) source_pad_blocked_cb, dpad);
     GST_DEBUG_OBJECT (dpad->pad, "unblocked");
-
   }
 
   group->dbin->activegroup = group;
@@ -1843,9 +1995,6 @@ gst_decode_group_expose (GstDecodeGroup * group)
 
   group->exposed = TRUE;
 
-  GST_LOG_OBJECT (group->dbin, "signalling no-more-pads");
-  gst_element_no_more_pads (GST_ELEMENT (group->dbin));
-
   GST_LOG_OBJECT (group->dbin, "Group %p exposed", group);
   return TRUE;
 }
@@ -1948,6 +2097,16 @@ gst_decode_group_free (GstDecodeGroup * group)
   GST_LOG ("group %p", group);
 
   GROUP_MUTEX_LOCK (group);
+
+  /* free ghost pads */
+  if (group == group->dbin->activegroup) {
+    for (tmp = group->ghosts; tmp; tmp = g_list_next (tmp))
+      gst_element_remove_pad (GST_ELEMENT (group->dbin), (GstPad *) tmp->data);
+
+    g_list_free (group->ghosts);
+    group->ghosts = NULL;
+  }
+
   /* Clear all GstDecodePad */
   for (tmp = group->endpads; tmp; tmp = g_list_next (tmp)) {
     GstDecodePad *dpad = (GstDecodePad *) tmp->data;
@@ -1957,6 +2116,13 @@ gst_decode_group_free (GstDecodeGroup * group)
   g_list_free (group->endpads);
   group->endpads = NULL;
 
+  /* release request pads */
+  for (tmp = group->reqpads; tmp; tmp = g_list_next (tmp)) {
+    gst_element_release_request_pad (group->multiqueue, GST_PAD (tmp->data));
+  }
+  g_list_free (group->reqpads);
+  group->reqpads = NULL;
+
   /* disconnect signal handlers on multiqueue */
   if (group->multiqueue) {
     if (group->underrunsig)
@@ -2170,6 +2336,7 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
         goto missing_typefind;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:{
+      dbin->have_type = FALSE;
       if (!add_fakesink (dbin))
         goto missing_fakesink;
       break;
