@@ -35,6 +35,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_play_base_bin_debug);
 #define DEFAULT_QUEUE_SIZE          (3 * GST_SECOND)
 #define DEFAULT_QUEUE_MIN_THRESHOLD ((DEFAULT_QUEUE_SIZE * 30) / 100)
 #define DEFAULT_QUEUE_THRESHOLD     ((DEFAULT_QUEUE_SIZE * 95) / 100)
+#define DEFAULT_CONNECTION_SPEED    0
 
 #define GROUP_LOCK(pbb) g_mutex_lock (pbb->group_lock)
 #define GROUP_UNLOCK(pbb) g_mutex_unlock (pbb->group_lock)
@@ -57,7 +58,8 @@ enum
   ARG_VIDEO,
   ARG_AUDIO,
   ARG_TEXT,
-  ARG_SUBTITLE_ENCODING
+  ARG_SUBTITLE_ENCODING,
+  ARG_CONNECTION_SPEED
 };
 
 static void gst_play_base_bin_class_init (GstPlayBaseBinClass * klass);
@@ -80,6 +82,9 @@ static GValueArray *gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin
     * play_base_bin);
 static void preroll_remove_overrun (GstElement * element,
     GstPlayBaseBin * play_base_bin);
+static void queue_remove_probe (GstElement * queue, GstPlayBaseBin
+    * play_base_bin);
+
 static GstElement *make_decoder (GstPlayBaseBin * play_base_bin);
 static gboolean has_all_raw_caps (GstPad * pad, gboolean * all_raw);
 
@@ -191,6 +196,17 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
           "If not set, the GST_SUBTITLE_ENCODING environment variable will "
           "be checked for an encoding to use. If that is not set either, "
           "ISO-8859-15 will be assumed.", NULL, G_PARAM_READWRITE));
+  /**
+   * GstPlayBin::connection-speed
+   *
+   * Network connection speed in kbps (0 = unknown)
+   *
+   * Since: 0.10.10 at gstplaybin.c, 0.10.15 moved to gstplaybasebin
+   **/
+  g_object_class_install_property (gobject_klass, ARG_CONNECTION_SPEED,
+      g_param_spec_uint ("connection-speed", "Connection Speed",
+          "Network connection speed in kbps (0 = unknown)",
+          0, G_MAXUINT, DEFAULT_CONNECTION_SPEED, G_PARAM_READWRITE));
 
   GST_DEBUG_CATEGORY_INIT (gst_play_base_bin_debug, "playbasebin", 0,
       "playbasebin");
@@ -228,6 +244,7 @@ gst_play_base_bin_init (GstPlayBaseBin * play_base_bin)
   play_base_bin->queue_size = DEFAULT_QUEUE_SIZE;
   play_base_bin->queue_threshold = DEFAULT_QUEUE_THRESHOLD;
   play_base_bin->queue_min_threshold = DEFAULT_QUEUE_MIN_THRESHOLD;
+  play_base_bin->connection_speed = DEFAULT_CONNECTION_SPEED;
 }
 
 static void
@@ -438,6 +455,8 @@ group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal, gboolean subtitle)
           continue;
 
         preroll_remove_overrun (element, play_base_bin);
+        /* if overrun is removed, probe alse has to be removed */
+        queue_remove_probe (element, play_base_bin);
       }
     } else {
       /* this is a special subtitle bin, we don't commit the group but
@@ -597,13 +616,39 @@ queue_deadlock_check (GstElement * queue, GstPlayBaseBin * play_base_bin)
   }
 }
 
+static void
+queue_remove_probe (GstElement * queue, GstPlayBaseBin * play_base_bin)
+{
+  gpointer data;
+  GstPad *sinkpad;
+
+  data = g_object_get_data (G_OBJECT (queue), "probe");
+  sinkpad = gst_element_get_pad (queue, "sink");
+
+  if (data) {
+    GST_DEBUG_OBJECT (play_base_bin,
+        "Removing buffer probe from pad %s:%s (%p)",
+        GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+
+    g_object_set_data (G_OBJECT (queue), "probe", NULL);
+    gst_pad_remove_buffer_probe (sinkpad, GPOINTER_TO_INT (data));
+  } else {
+    GST_DEBUG_OBJECT (play_base_bin,
+        "No buffer probe to remove from %s:%s (%p)",
+        GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+  }
+  gst_object_unref (sinkpad);
+}
+
 /* Used for time-based buffering in streaming mode and is called when a queue
  * emits the running signal. This means that the high watermark threshold is
  * reached and the buffering is completed. */
 static void
 queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
 {
+  GstPlayBaseGroup *group;
   gpointer data;
+  gint n;
 
   GST_DEBUG_OBJECT (play_base_bin, "running signal received from queue %s",
       GST_ELEMENT_NAME (queue));
@@ -626,26 +671,30 @@ queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
         play_base_bin->queue_min_threshold, NULL);
   }
 
+  GROUP_LOCK (play_base_bin);
+  group = get_active_group (play_base_bin);
+  if (!group) {
+    GROUP_UNLOCK (play_base_bin);
+    return;
+  }
+
   /* we remove the probe now because we don't need it anymore to give progress
    * about the buffering. */
-  data = g_object_get_data (G_OBJECT (queue), "probe");
-  if (data) {
-    GstPad *sinkpad;
+  for (n = 0; n < NUM_TYPES; n++) {
+    GstElement *element = group->type[n].preroll;
 
-    sinkpad = gst_element_get_pad (queue, "sink");
-    GST_DEBUG_OBJECT (play_base_bin,
-        "Removing buffer probe from pad %s:%s (%p)",
-        GST_DEBUG_PAD_NAME (sinkpad), sinkpad);
+    if (!element)
+      continue;
 
-    g_object_set_data (G_OBJECT (queue), "probe", NULL);
-    gst_pad_remove_buffer_probe (sinkpad, GPOINTER_TO_INT (data));
-
-    gst_object_unref (sinkpad);
+    queue_remove_probe (element, play_base_bin);
   }
 
+  GROUP_UNLOCK (play_base_bin);
+
   /* we post a 100% buffering message to notify the app that buffering is
    * completed and playback can start/continue */
-  fill_buffer (play_base_bin, 100);
+  if (play_base_bin->is_stream)
+    fill_buffer (play_base_bin, 100);
 }
 
 /* this signal will be fired when one of the queues with raw
@@ -768,7 +817,7 @@ gen_preroll_element (GstPlayBaseBin * play_base_bin,
    */
   g_object_set (G_OBJECT (preroll),
       "max-size-buffers", 0, "max-size-bytes",
-      ((type == GST_STREAM_TYPE_VIDEO) ? 25 : 1) * 1024 * 1024,
+      ((type == GST_STREAM_TYPE_VIDEO) ? 25 : 2) * 1024 * 1024,
       "max-size-time", play_base_bin->queue_size, NULL);
 
   /* the overrun signal is always attached and serves two purposes:
@@ -1501,7 +1550,7 @@ array_has_value (const gchar * values[], const gchar * value)
 /* list of URIs that we consider to be streams and that need buffering.
  * We have no mechanism yet to figure this out with a query. */
 static const gchar *stream_uris[] = { "http://", "mms://", "mmsh://",
-  "mmsu://", "mmst://", NULL
+  "mmsu://", "mmst://", "myth://", NULL
 };
 
 /* blacklisted URIs, we know they will always fail. */
@@ -1564,6 +1613,16 @@ gen_source_element (GstPlayBaseBin * play_base_bin, GstElement ** subbin)
           "iradio-mode")) {
     g_object_set (source, "iradio-mode", TRUE, NULL);
   }
+
+  if (g_object_class_find_property (G_OBJECT_GET_CLASS (source),
+          "connection-speed")) {
+    GST_DEBUG_OBJECT (play_base_bin,
+        "setting connection-speed=%d to source element",
+        play_base_bin->connection_speed / 1000);
+    g_object_set (source, "connection-speed",
+        play_base_bin->connection_speed / 1000, NULL);
+  }
+
   return source;
 
   /* ERRORS */
@@ -1668,6 +1727,8 @@ setup_substreams (GstPlayBaseBin * play_base_bin)
   gint n;
   const GList *item;
 
+  GST_DEBUG_OBJECT (play_base_bin, "setting up substreams");
+
   /* Remove the eat probes */
   group = get_active_group (play_base_bin);
   for (item = group->streaminfo; item; item = item->next) {
@@ -1699,6 +1760,7 @@ setup_substreams (GstPlayBaseBin * play_base_bin)
    * we have output. Always keep it enabled. */
   for (n = 0; n < NUM_TYPES; n++) {
     if (play_base_bin->current[n] >= group->type[n].npads) {
+      GST_DEBUG_OBJECT (play_base_bin, "reset type %d to current 0", n);
       play_base_bin->current[n] = 0;
     }
   }
@@ -1706,6 +1768,8 @@ setup_substreams (GstPlayBaseBin * play_base_bin)
   /* now activate the right sources. Don't forget that during preroll,
    * we set the first source to forwarding and ignored the rest. */
   for (n = 0; n < NUM_TYPES; n++) {
+    GST_DEBUG_OBJECT (play_base_bin, "setting type %d to current %d", n,
+        play_base_bin->current[n]);
     set_active_source (play_base_bin, n + 1, play_base_bin->current[n]);
   }
 }
@@ -2149,6 +2213,7 @@ could_not_link:
 static void
 finish_source (GstPlayBaseBin * play_base_bin)
 {
+  /* FIXME: no need to grab the group lock here? (tpm) */
   if (get_active_group (play_base_bin) != NULL) {
     if (play_base_bin->subtitle) {
       /* make subs iterate from now on */
@@ -2436,6 +2501,9 @@ gst_play_base_bin_set_property (GObject * object, guint prop_id,
     case ARG_QUEUE_MIN_THRESHOLD:
       play_base_bin->queue_min_threshold = g_value_get_uint64 (value);
       break;
+    case ARG_CONNECTION_SPEED:
+      play_base_bin->connection_speed = g_value_get_uint (value) * 1000;
+      break;
     case ARG_VIDEO:
       GROUP_LOCK (play_base_bin);
       set_active_source (play_base_bin,
@@ -2528,6 +2596,9 @@ gst_play_base_bin_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_QUEUE_MIN_THRESHOLD:
       g_value_set_uint64 (value, play_base_bin->queue_min_threshold);
       break;
+    case ARG_CONNECTION_SPEED:
+      g_value_set_uint (value, play_base_bin->connection_speed / 1000);
+      break;
     case ARG_STREAMINFO:
       /* FIXME: hold some kind of lock here, use iterator */
       g_value_set_pointer (value,
