@@ -50,7 +50,7 @@ G_STMT_START {                                 \
 } G_STMT_END
 
 
-GST_DEBUG_CATEGORY (tcpclientsrc_debug);
+GST_DEBUG_CATEGORY_STATIC (tcpclientsrc_debug);
 #define GST_CAT_DEFAULT tcpclientsrc_debug
 
 #define MAX_READ_SIZE                   4 * 1024
@@ -123,11 +123,11 @@ gst_tcp_client_src_class_init (GstTCPClientSrcClass * klass)
   gobject_class->get_property = gst_tcp_client_src_get_property;
   gobject_class->finalize = gst_tcp_client_src_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_HOST,
+  g_object_class_install_property (gobject_class, PROP_HOST,
       g_param_spec_string ("host", "Host",
           "The host IP address to receive packets from", TCP_DEFAULT_HOST,
           G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
+  g_object_class_install_property (gobject_class, PROP_PORT,
       g_param_spec_int ("port", "Port", "The port to receive packets from", 0,
           TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_PROTOCOL,
@@ -168,6 +168,8 @@ gst_tcp_client_src_finalize (GObject * gobject)
   GstTCPClientSrc *this = GST_TCP_CLIENT_SRC (gobject);
 
   g_free (this->host);
+
+  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static GstCaps *
