@@ -40,7 +40,7 @@ GST_DEBUG_CATEGORY_STATIC (type_find_debug);
 /*** text/plain ***/
 static gboolean xml_check_first_element (GstTypeFind * tf,
     const gchar * element, guint elen, gboolean strict);
-
+static gboolean sdp_check_header (GstTypeFind * tf);
 
 static GstStaticCaps utf8_caps = GST_STATIC_CAPS ("text/plain");
 
@@ -89,6 +89,10 @@ utf8_type_find (GstTypeFind * tf, gpointer unused)
   if (xml_check_first_element (tf, "", 0, TRUE))
     return;
 
+  /* leave sdp to the sdp typefinders */
+  if (sdp_check_header (tf))
+    return;
+
   /* check beginning of stream */
   if (!utf8_type_find_have_valid_utf8_at_offset (tf, 0, &start_prob))
     return;
@@ -259,6 +263,39 @@ xml_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/sdp *********************************************************/
+
+static GstStaticCaps sdp_caps = GST_STATIC_CAPS ("application/sdp");
+
+#define SDP_CAPS (gst_static_caps_get(&sdp_caps))
+static gboolean
+sdp_check_header (GstTypeFind * tf)
+{
+  guint8 *data;
+
+  data = gst_type_find_peek (tf, 0, 5);
+  if (!data)
+    return FALSE;
+
+  /* sdp must start with v=0[\r]\n */
+  if (memcmp (data, "v=0", 3))
+    return FALSE;
+
+  if (data[3] == '\r' && data[4] != '\n')
+    return FALSE;
+  if (data[3] != '\n')
+    return FALSE;
+
+  return TRUE;
+}
+
+static void
+sdp_type_find (GstTypeFind * tf, gpointer unused)
+{
+  if (sdp_check_header (tf))
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SDP_CAPS);
+}
+
 /*** application/smil *********************************************************/
 
 static GstStaticCaps smil_caps = GST_STATIC_CAPS ("application/smil");
@@ -2738,7 +2775,6 @@ G_BEGIN_DECLS{                                                          \
 }G_END_DECLS
 
 
-
 /*** plugin initialization ***/
 
 #define TYPE_FIND_REGISTER(plugin,name,rank,func,ext,caps,priv,notify) \
@@ -2746,6 +2782,8 @@ G_BEGIN_DECLS{\
   if (!gst_type_find_register (plugin, name, rank, func, ext, caps, priv, notify))\
     return FALSE; \
 }G_END_DECLS
+
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -2759,6 +2797,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *asf_exts[] = { "asf", "wm", "wma", "wmv", NULL };
   static gchar *au_exts[] = { "au", "snd", NULL };
   static gchar *avi_exts[] = { "avi", NULL };
+  static gchar *qcp_exts[] = { "qcp", NULL };
   static gchar *cdxa_exts[] = { "dat", NULL };
   static gchar *flac_exts[] = { "flac", NULL };
   static gchar *flx_exts[] = { "flc", "fli", NULL };
@@ -2794,6 +2833,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *shn_exts[] = { "shn", NULL };
   static gchar *ape_exts[] = { "ape", NULL };
   static gchar *uri_exts[] = { "ram", NULL };
+  static gchar *sdp_exts[] = { "sdp", NULL };
   static gchar *smil_exts[] = { "smil", NULL };
   static gchar *html_exts[] = { "htm", "html", NULL };
   static gchar *xml_exts[] = { "xml", NULL };
@@ -2833,6 +2873,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *m4v_exts[] = { "m4v", NULL };
   static gchar *nuv_exts[] = { "nuv", NULL };
   static gchar *vivo_exts[] = { "viv", NULL };
+  static gchar *nsf_exts[] = { "nsf", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
       GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
@@ -2848,6 +2889,8 @@ plugin_init (GstPlugin * plugin)
       au_type_find, au_exts, AU_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
       avi_exts, "AVI ");
+  TYPE_FIND_REGISTER_RIFF (plugin, "audio/qcelp", GST_RANK_PRIMARY,
+      qcp_exts, "QLCM");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
       cdxa_exts, "CDXA");
   TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-vcd", GST_RANK_PRIMARY,
@@ -2909,6 +2952,8 @@ plugin_init (GstPlugin * plugin)
       utf8_exts, UTF8_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "text/uri-list", GST_RANK_MARGINAL, uri_type_find,
       uri_exts, URI_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/sdp", GST_RANK_SECONDARY,
+      sdp_type_find, sdp_exts, SDP_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/smil", GST_RANK_SECONDARY,
       smil_type_find, smil_exts, SMIL_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/xml", GST_RANK_MARGINAL,
@@ -3018,16 +3063,14 @@ plugin_init (GstPlugin * plugin)
       GST_RANK_SECONDARY, msdos_type_find, msdos_exts, MSDOS_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "video/x-dirac", GST_RANK_PRIMARY,
       dirac_type_find, NULL, DIRAC_CAPS, NULL, NULL);
-#if 0
-  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
-      GST_RANK_PRIMARY, NULL, "BBCD", 4, GST_TYPE_FIND_MAXIMUM);
-#endif
   TYPE_FIND_REGISTER (plugin, "multipart/x-mixed-replace", GST_RANK_SECONDARY,
       multipart_type_find, NULL, MULTIPART_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-mmsh", GST_RANK_SECONDARY,
       mmsh_type_find, NULL, MMSH_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "video/vivo", GST_RANK_SECONDARY,
       vivo_type_find, vivo_exts, VIVO_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-nsf",
+      GST_RANK_SECONDARY, nsf_exts, "NESM\x1a", 5, GST_TYPE_FIND_MAXIMUM);
   return TRUE;
 }
 
