@@ -25,10 +25,10 @@
  * SECTION:element-volume
  *
  * <refsect2>
- * <title>Example launch line</title>
  * <para>
  * The volume element changes the volume of the audio data.
  * </para>
+ * <title>Example launch line</title>
  * <para>
  * <programlisting>
  * gst-launch -v -m audiotestsrc ! volume volume=0.5 ! level ! fakesink silent=TRUE
@@ -55,14 +55,26 @@
 #include "gstvolume.h"
 
 /* some defines for audio processing */
-/* the volume factor is a range from 0.0 to (arbitrary) 4.0
- * we map 1.0 to VOLUME_UNITY_INT
+/* the volume factor is a range from 0.0 to (arbitrary) VOLUME_MAX_DOUBLE = 10.0
+ * we map 1.0 to VOLUME_UNITY_INT*
  */
-#define VOLUME_UNITY_INT        8192    /* internal int for unity */
-#define VOLUME_UNITY_BIT_SHIFT  13      /* number of bits to shift for unity */
-#define VOLUME_MAX_DOUBLE       10.0
-#define VOLUME_MAX_INT16        32767
-#define VOLUME_MIN_INT16        -32768
+#define VOLUME_UNITY_INT8            32 /* internal int for unity 2^(8-3) */
+#define VOLUME_UNITY_INT8_BIT_SHIFT  5  /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT16           8192       /* internal int for unity 2^(16-3) */
+#define VOLUME_UNITY_INT16_BIT_SHIFT 13 /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT24           2097152    /* internal int for unity 2^(24-3) */
+#define VOLUME_UNITY_INT24_BIT_SHIFT 21 /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT32           134217728  /* internal int for unity 2^(32-5) */
+#define VOLUME_UNITY_INT32_BIT_SHIFT 27
+#define VOLUME_MAX_DOUBLE            10.0
+#define VOLUME_MAX_INT8              G_MAXINT8
+#define VOLUME_MIN_INT8              G_MININT8
+#define VOLUME_MAX_INT16             G_MAXINT16
+#define VOLUME_MIN_INT16             G_MININT16
+#define VOLUME_MAX_INT24             8388607
+#define VOLUME_MIN_INT24             -8388608
+#define VOLUME_MAX_INT32             G_MAXINT32
+#define VOLUME_MIN_INT32             G_MININT32
 
 /* number of steps we use for the mixer interface to go from 0.0 to 1.0 */
 # define VOLUME_STEPS           100
@@ -98,12 +110,33 @@ static GstStaticPadTemplate volume_sink_template =
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) 32; "
+        "width = (int) {32, 64}; "
         "audio/x-raw-int, "
         "channels = (int) [ 1, MAX ], "
         "rate = (int) [ 1,  MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) 16, " "depth = (int) 16, " "signed = (bool) TRUE")
+        "width = (int) 8, "
+        "depth = (int) 8, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 16, "
+        "depth = (int) 16, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 24, "
+        "depth = (int) 24, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 32, " "depth = (int) 32, " "signed = (bool) TRUE")
     );
 
 static GstStaticPadTemplate volume_src_template = GST_STATIC_PAD_TEMPLATE
@@ -114,12 +147,33 @@ static GstStaticPadTemplate volume_src_template = GST_STATIC_PAD_TEMPLATE
         "rate = (int) [ 1, MAX ], "
         "channels = (int) [ 1, MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) 32; "
+        "width = (int) {32, 64}; "
         "audio/x-raw-int, "
         "channels = (int) [ 1, MAX ], "
         "rate = (int) [ 1,  MAX ], "
         "endianness = (int) BYTE_ORDER, "
-        "width = (int) 16, " "depth = (int) 16, " "signed = (bool) TRUE")
+        "width = (int) 8, "
+        "depth = (int) 8, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 16, "
+        "depth = (int) 16, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 24, "
+        "depth = (int) 24, "
+        "signed = (bool) TRUE; "
+        "audio/x-raw-int, "
+        "channels = (int) [ 1, MAX ], "
+        "rate = (int) [ 1,  MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) 32, " "depth = (int) 32, " "signed = (bool) TRUE")
     );
 
 static void gst_volume_interface_init (GstImplementsInterfaceClass * klass);
@@ -127,12 +181,12 @@ static void gst_volume_mixer_init (GstMixerClass * iface);
 
 #define _init_interfaces(type)                                          \
   {                                                                     \
-    static const GInterfaceInfo voliface_info = {                       \
+    static const GInterfaceInfo voliface_info = {                     \
       (GInterfaceInitFunc) gst_volume_interface_init,                   \
       NULL,                                                             \
       NULL                                                              \
     };                                                                  \
-    static const GInterfaceInfo volmixer_info = {                       \
+    static const GInterfaceInfo volmixer_info = {                     \
       (GInterfaceInitFunc) gst_volume_mixer_init,                       \
       NULL,                                                             \
       NULL                                                              \
@@ -158,33 +212,87 @@ static GstFlowReturn volume_transform_ip (GstBaseTransform * base,
 static gboolean volume_set_caps (GstBaseTransform * base, GstCaps * incaps,
     GstCaps * outcaps);
 
+static void volume_process_double (GstVolume * this, gpointer bytes,
+    guint n_bytes);
 static void volume_process_float (GstVolume * this, gpointer bytes,
-    gint n_bytes);
+    guint n_bytes);
+static void volume_process_int32 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int32_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int24 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int24_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
 static void volume_process_int16 (GstVolume * this, gpointer bytes,
-    gint n_bytes);
+    guint n_bytes);
 static void volume_process_int16_clamp (GstVolume * this, gpointer bytes,
-    gint n_bytes);
+    guint n_bytes);
+static void volume_process_int8 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int8_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+
 
 /* helper functions */
 
 static void
 volume_choose_func (GstVolume * this)
 {
+  this->process = NULL;
   switch (this->format) {
     case GST_VOLUME_FORMAT_INT:
-      /* only clamp if the gain is greater than 1.0
-       * FIXME: real_vol_i can change while processing the buffer!
-       */
-      if (this->real_vol_i > VOLUME_UNITY_INT)
-        this->process = volume_process_int16_clamp;
-      else
-        this->process = volume_process_int16;
+      switch (this->width) {
+        case 32:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i32 > VOLUME_UNITY_INT32)
+            this->process = volume_process_int32_clamp;
+          else
+            this->process = volume_process_int32;
+          break;
+        case 24:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i24 > VOLUME_UNITY_INT24)
+            this->process = volume_process_int24_clamp;
+          else
+            this->process = volume_process_int24;
+          break;
+        case 16:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i16 > VOLUME_UNITY_INT16)
+            this->process = volume_process_int16_clamp;
+          else
+            this->process = volume_process_int16;
+          break;
+        case 8:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i16 > VOLUME_UNITY_INT8)
+            this->process = volume_process_int8_clamp;
+          else
+            this->process = volume_process_int8;
+          break;
+      }
       break;
     case GST_VOLUME_FORMAT_FLOAT:
-      this->process = volume_process_float;
+      switch (this->width) {
+        case 32:
+          this->process = volume_process_float;
+          break;
+        case 64:
+          this->process = volume_process_double;
+          break;
+      }
       break;
     default:
-      this->process = NULL;
+      break;
   }
 }
 
@@ -195,11 +303,15 @@ volume_update_real_volume (GstVolume * this)
 
   if (this->mute) {
     this->real_vol_f = 0.0;
-    this->real_vol_i = 0;
+    this->real_vol_i8 = this->real_vol_i16 = this->real_vol_i24 =
+        this->real_vol_i32 = 0;
   } else {
     this->real_vol_f = this->volume_f;
-    this->real_vol_i = this->volume_i;
-    passthrough = (this->volume_i == VOLUME_UNITY_INT);
+    this->real_vol_i8 = this->volume_i8;
+    this->real_vol_i16 = this->volume_i16;
+    this->real_vol_i24 = this->volume_i24;
+    this->real_vol_i32 = this->volume_i32;
+    passthrough = (this->volume_i16 == VOLUME_UNITY_INT16);
   }
   volume_choose_func (this);
   gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (this), passthrough);
@@ -240,7 +352,10 @@ gst_volume_set_volume (GstMixer * mixer, GstMixerTrack * track, gint * volumes)
   g_return_if_fail (GST_IS_VOLUME (this));
 
   this->volume_f = (gfloat) volumes[0] / VOLUME_STEPS;
-  this->volume_i = this->volume_f * VOLUME_UNITY_INT;
+  this->volume_i32 = this->volume_f * VOLUME_UNITY_INT32;
+  this->volume_i24 = this->volume_f * VOLUME_UNITY_INT24;
+  this->volume_i16 = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i8 = this->volume_f * VOLUME_UNITY_INT8;
 
   volume_update_real_volume (this);
 }
@@ -344,10 +459,11 @@ gst_volume_init (GstVolume * this, GstVolumeClass * g_class)
   GstMixerTrack *track = NULL;
 
   this->mute = FALSE;
-  this->volume_i = VOLUME_UNITY_INT;
-  this->volume_f = 1.0;
-  this->real_vol_i = VOLUME_UNITY_INT;
-  this->real_vol_f = 1.0;
+  this->volume_i8 = this->real_vol_i8 = VOLUME_UNITY_INT8;
+  this->volume_i16 = this->real_vol_i16 = VOLUME_UNITY_INT16;
+  this->volume_i24 = this->real_vol_i24 = VOLUME_UNITY_INT24;
+  this->volume_i32 = this->real_vol_i32 = VOLUME_UNITY_INT32;
+  this->volume_f = this->real_vol_f = 1.0;
   this->tracklist = NULL;
   this->format = GST_VOLUME_FORMAT_NONE;
 
@@ -365,65 +481,204 @@ gst_volume_init (GstVolume * this, GstVolumeClass * g_class)
 
 /* NOTE: although it might be tempting to have volume_process_mute() which uses
  *       memset(bytes, 0, nbytes) for the vol=0 case, this has the downside that
- *       unmuting would unly take place after processing a buffer.
+ *       unmuting would only take place after processing a buffer.
  */
 
 static void
-volume_process_float (GstVolume * this, gpointer bytes, gint n_bytes)
+volume_process_double (GstVolume * this, gpointer bytes, guint n_bytes)
 {
-  gfloat *data;
-  gint i, num_samples;
-
-  data = (gfloat *) bytes;
-  num_samples = n_bytes / sizeof (gfloat);
+  gdouble *data = (gdouble *) bytes;
+  guint i, num_samples = n_bytes / sizeof (gdouble);
 
   for (i = 0; i < num_samples; i++) {
     *data++ *= this->real_vol_f;
   }
-  /* FIXME: seems to be slower than above!
-     oil_scalarmultiply_f32_ns (data, data, &this->real_vol_f, num_samples);
-   */
 }
 
 static void
-volume_process_int16 (GstVolume * this, gpointer bytes, gint n_bytes)
+volume_process_float (GstVolume * this, gpointer bytes, guint n_bytes)
 {
-  gint16 *data;
-  gint i, val, num_samples;
+  gfloat *data = (gfloat *) bytes;
+  guint num_samples = n_bytes / sizeof (gfloat);
 
-  data = (gint16 *) bytes;
-  num_samples = n_bytes / sizeof (gint16);
-
-  /* FIXME: need... liboil... 
-   * oil_scalarmultiply_s16_ns ?
-   * https://bugs.freedesktop.org/show_bug.cgi?id=7060
+  /*
+     guint i;
+     for (i = 0; i < num_samples; i++) {
+     *data++ *= this->real_vol_f;
+     }
    */
+  /* time gst-launch 2>/dev/null audiotestsrc wave=7 num-buffers=10000 ! audio/x-raw-float ! volume volume=1.5 ! fakesink
+   * goes from 0m0.850s -> 0m0.717s with liboil
+   */
+  oil_scalarmultiply_f32_ns (data, data, &this->real_vol_f, num_samples);
+}
+
+static void
+volume_process_int32 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint *data = (gint *) bytes;
+  guint i, num_samples;
+  gint64 val;
+
+  num_samples = n_bytes / sizeof (gint);
   for (i = 0; i < num_samples; i++) {
     /* we use bitshifting instead of dividing by UNITY_INT for speed */
-    val = (gint) * data;
-    *data++ = (gint16) ((this->real_vol_i * val) >> VOLUME_UNITY_BIT_SHIFT);
+    val = (gint64) * data;
+    val = (((gint64) this->real_vol_i32 * val) >> VOLUME_UNITY_INT32_BIT_SHIFT);
+    *data++ = (gint32) val;
   }
 }
 
 static void
-volume_process_int16_clamp (GstVolume * this, gpointer bytes, gint n_bytes)
+volume_process_int32_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
 {
-  gint16 *data;
-  gint i, val, num_samples;
+  gint *data = (gint *) bytes;
+  guint i, num_samples;
+  gint64 val;
+
+  num_samples = n_bytes / sizeof (gint);
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint64) * data;
+    val = (((gint64) this->real_vol_i32 * val) >> VOLUME_UNITY_INT32_BIT_SHIFT);
+    *data++ = (gint32) CLAMP (val, VOLUME_MIN_INT32, VOLUME_MAX_INT32);
+  }
+}
+
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+#define get_unaligned_i24(_x) ( (((guint8*)_x)[0]) | ((((guint8*)_x)[1]) << 8) | ((((gint8*)_x)[2]) << 16) )
+#define write_unaligned_u24(_x,samp) do { *(_x)++ = samp & 0xFF; *(_x)++ = (samp >> 8) & 0xFF; *(_x)++ = (samp >> 16) & 0xFF; } while (0)
+#else /* BIG ENDIAN */
+#define get_unaligned_i24(_x) ( (((guint8*)_x)[2]) | ((((guint8*)_x)[1]) << 8) | ((((gint8*)_x)[0]) << 16) )
+#define write_unaligned_u24(_x,samp) do { *(_x)++ = (samp >> 16) & 0xFF; *(_x)++ = (samp >> 8) & 0xFF; *(_x)++ = samp & 0xFF; } while (0)
+#endif
+
+static void
+volume_process_int24 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;        /* treat the data as a byte stream */
+  guint i, num_samples;
+  guint32 samp;
+  gint64 val;
+
+  num_samples = n_bytes / (sizeof (gint8) * 3);
+  for (i = 0; i < num_samples; i++) {
+    samp = get_unaligned_i24 (data);
+
+    val = (gint32) samp;
+    val = (((gint64) this->real_vol_i24 * val) >> VOLUME_UNITY_INT24_BIT_SHIFT);
+    samp = (guint32) val;
+
+    /* write the value back into the stream */
+    write_unaligned_u24 (data, samp);
+  }
+}
+
+static void
+volume_process_int24_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;        /* treat the data as a byte stream */
+  guint i, num_samples;
+  guint32 samp;
+  gint64 val;
+
+  num_samples = n_bytes / (sizeof (gint8) * 3);
+  for (i = 0; i < num_samples; i++) {
+    samp = get_unaligned_i24 (data);
+
+    val = (gint32) samp;
+    val = (((gint64) this->real_vol_i24 * val) >> VOLUME_UNITY_INT24_BIT_SHIFT);
+    samp = (guint32) CLAMP (val, VOLUME_MIN_INT24, VOLUME_MAX_INT24);
+
+    /* write the value back into the stream */
+    write_unaligned_u24 (data, samp);
+  }
+}
+
+static void
+volume_process_int16 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint16 *data = (gint16 *) bytes;
+  guint num_samples = n_bytes / sizeof (gint16);
+
+#if 1
+  guint i;
+  gint val;
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint) * data;
+    *data++ =
+        (gint16) ((this->real_vol_i16 * val) >> VOLUME_UNITY_INT16_BIT_SHIFT);
+  }
+#else
+  /* FIXME: need oil_scalarmultiply_s16_ns ?
+   * https://bugs.freedesktop.org/show_bug.cgi?id=7060
+   * code below
+   * - crashes :/
+   * - real_vol_i is scaled by VOLUME_UNITY_INT16 and needs the bitshift
+   * time gst-launch 2>/dev/null audiotestsrc wave=7 num-buffers=100 ! volume volume=1.5 ! fakesink
+   */
+  oil_scalarmult_s16 (data, 0, data, 0,
+      ((gint16 *) (void *) (&this->real_vol_i)), num_samples);
+#endif
+}
+
+static void
+volume_process_int16_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint16 *data = (gint16 *) bytes;
+  guint i, num_samples;
+  gint val;
 
-  data = (gint16 *) bytes;
   num_samples = n_bytes / sizeof (gint16);
 
-  /* FIXME: need... liboil... 
-   * oil_scalarmultiply_s16_ns ?
+  /* FIXME: oil_scalarmultiply_s16_ns ?
    * https://bugs.freedesktop.org/show_bug.cgi?id=7060
    */
   for (i = 0; i < num_samples; i++) {
     /* we use bitshifting instead of dividing by UNITY_INT for speed */
     val = (gint) * data;
     *data++ =
-        (gint16) CLAMP ((this->real_vol_i * val) >> VOLUME_UNITY_BIT_SHIFT,
-        VOLUME_MIN_INT16, VOLUME_MAX_INT16);
+        (gint16) CLAMP ((this->real_vol_i16 *
+            val) >> VOLUME_UNITY_INT16_BIT_SHIFT, VOLUME_MIN_INT16,
+        VOLUME_MAX_INT16);
+  }
+}
+
+static void
+volume_process_int8 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;
+  guint num_samples = n_bytes / sizeof (gint8);
+  guint i;
+  gint val;
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint) * data;
+    *data++ =
+        (gint8) ((this->real_vol_i8 * val) >> VOLUME_UNITY_INT8_BIT_SHIFT);
+  }
+}
+
+static void
+volume_process_int8_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;
+  guint i, num_samples;
+  gint val;
+
+  num_samples = n_bytes / sizeof (gint8);
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint) * data;
+    *data++ =
+        (gint8) CLAMP ((this->real_vol_i8 *
+            val) >> VOLUME_UNITY_INT8_BIT_SHIFT, VOLUME_MIN_INT8,
+        VOLUME_MAX_INT8);
   }
 }
 
@@ -435,19 +690,22 @@ volume_set_caps (GstBaseTransform * base, GstCaps * incaps, GstCaps * outcaps)
 {
   GstVolume *this = GST_VOLUME (base);
   const gchar *mimetype;
+  GstStructure *structure;
 
   GST_DEBUG_OBJECT (this,
       "set_caps: in %" GST_PTR_FORMAT " out %" GST_PTR_FORMAT, incaps, outcaps);
 
-  mimetype = gst_structure_get_name (gst_caps_get_structure (incaps, 0));
+  structure = gst_caps_get_structure (incaps, 0);
+  gst_structure_get_int (structure, "width", &this->width);
+  mimetype = gst_structure_get_name (structure);
 
   /* based on mimetype, choose the correct volume_process format */
   if (strcmp (mimetype, "audio/x-raw-int") == 0) {
     this->format = GST_VOLUME_FORMAT_INT;
-    GST_INFO_OBJECT (this, "use int16");
+    GST_DEBUG_OBJECT (this, "use int: %u", this->width);
   } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
     this->format = GST_VOLUME_FORMAT_FLOAT;
-    GST_INFO_OBJECT (this, "use float");
+    GST_DEBUG_OBJECT (this, "use float: %u", this->width);
   } else {
     this->process = NULL;
     goto invalid_format;
@@ -475,11 +733,9 @@ volume_transform_ip (GstBaseTransform * base, GstBuffer * outbuf)
   GstVolume *this = GST_VOLUME (base);
   GstClockTime timestamp;
 
-  /* don't process data in passthrough-mode */
-  if (gst_base_transform_is_passthrough (base))
-    return GST_FLOW_OK;
-
-  /* FIXME: subdivide GST_BUFFER_SIZE into small chunks for smooth fades */
+  /* FIXME: if controllers are bound, subdivide GST_BUFFER_SIZE into small
+   * chunks for smooth fades, what is small? 1/10th sec.
+   */
   timestamp = GST_BUFFER_TIMESTAMP (outbuf);
   timestamp =
       gst_segment_to_stream_time (&base->segment, GST_FORMAT_TIME, timestamp);
@@ -490,6 +746,10 @@ volume_transform_ip (GstBaseTransform * base, GstBuffer * outbuf)
   if (GST_CLOCK_TIME_IS_VALID (timestamp))
     gst_object_sync_values (G_OBJECT (this), timestamp);
 
+  /* don't process data in passthrough-mode */
+  if (gst_base_transform_is_passthrough (base))
+    return GST_FLOW_OK;
+
   this->process (this, GST_BUFFER_DATA (outbuf), GST_BUFFER_SIZE (outbuf));
 
   return GST_FLOW_OK;
@@ -519,7 +779,10 @@ volume_update_volume (const GValue * value, gpointer data)
   g_return_if_fail (GST_IS_VOLUME (this));
 
   this->volume_f = g_value_get_double (value);
-  this->volume_i = this->volume_f * VOLUME_UNITY_INT;
+  this->volume_i8 = this->volume_f * VOLUME_UNITY_INT8;
+  this->volume_i16 = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i24 = this->volume_f * VOLUME_UNITY_INT24;
+  this->volume_i32 = this->volume_f * VOLUME_UNITY_INT32;
 
   volume_update_real_volume (this);
 }
@@ -565,7 +828,7 @@ volume_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  /*oil_init (); */
+  oil_init ();
 
   /* initialize gst controller library */
   gst_controller_init (NULL, NULL);
