@@ -180,6 +180,8 @@ static GstVideoSinkClass *parent_class = NULL;
 
 /* ximage buffers */
 
+static GstBufferClass *ximage_buffer_parent_class = NULL;
+
 #define GST_TYPE_XIMAGE_BUFFER (gst_ximage_buffer_get_type())
 
 #define GST_IS_XIMAGE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_XIMAGE_BUFFER))
@@ -236,6 +238,10 @@ gst_ximage_buffer_finalize (GstXImageBuffer * ximage)
     recycled = TRUE;
   }
 
+  if (!recycled)
+    GST_MINI_OBJECT_CLASS (ximage_buffer_parent_class)->
+        finalize (GST_MINI_OBJECT (ximage));
+
 beach:
   return;
 }
@@ -263,6 +269,8 @@ gst_ximage_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  ximage_buffer_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_ximage_buffer_finalize;
 }
@@ -1081,12 +1089,18 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
 {
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
+  GST_OBJECT_LOCK (ximagesink);
   while (ximagesink->running) {
+    GST_OBJECT_UNLOCK (ximagesink);
+
     if (ximagesink->xwindow) {
       gst_ximagesink_handle_xevents (ximagesink);
     }
     g_usleep (100000);
+
+    GST_OBJECT_LOCK (ximagesink);
   }
+  GST_OBJECT_UNLOCK (ximagesink);
 
   return NULL;
 }
@@ -1271,8 +1285,11 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   g_mutex_unlock (ximagesink->x_lock);
 
   /* Setup our event listening thread */
+  GST_OBJECT_LOCK (ximagesink);
+  ximagesink->running = TRUE;
   ximagesink->event_thread = g_thread_create (
       (GThreadFunc) gst_ximagesink_event_thread, ximagesink, TRUE, NULL);
+  GST_OBJECT_UNLOCK (ximagesink);
 
   return xcontext;
 }
@@ -1300,12 +1317,6 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
 
   GST_OBJECT_UNLOCK (ximagesink);
 
-  /* Wait for our event thread */
-  if (ximagesink->event_thread) {
-    g_thread_join (ximagesink->event_thread);
-    ximagesink->event_thread = NULL;
-  }
-
   gst_caps_unref (xcontext->caps);
   g_free (xcontext->par);
   g_free (ximagesink->par);
@@ -1489,19 +1500,18 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
 
       /* Initializing the XContext */
-      if (!ximagesink->xcontext)
+      if (ximagesink->xcontext == NULL) {
         xcontext = gst_ximagesink_xcontext_get (ximagesink);
-
-      GST_OBJECT_LOCK (ximagesink);
-      ximagesink->running = TRUE;
-      if (xcontext)
-        ximagesink->xcontext = xcontext;
-      GST_OBJECT_UNLOCK (ximagesink);
-
-      if (!ximagesink->xcontext) {
-        ret = GST_STATE_CHANGE_FAILURE;
-        goto beach;
+        if (xcontext == NULL) {
+          ret = GST_STATE_CHANGE_FAILURE;
+          goto beach;
+        }
+        GST_OBJECT_LOCK (ximagesink);
+        if (xcontext)
+          ximagesink->xcontext = xcontext;
+        GST_OBJECT_UNLOCK (ximagesink);
       }
+
       /* call XSynchronize with the current value of synchronous */
       GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
           ximagesink->synchronous ? "TRUE" : "FALSE");
@@ -2089,10 +2099,19 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
 static void
 gst_ximagesink_reset (GstXImageSink * ximagesink)
 {
+  GThread *thread;
+
   GST_OBJECT_LOCK (ximagesink);
   ximagesink->running = FALSE;
+  /* grab thread and mark it as NULL */
+  thread = ximagesink->event_thread;
+  ximagesink->event_thread = NULL;
   GST_OBJECT_UNLOCK (ximagesink);
 
+  /* Wait for our event thread to finish before we clean up our stuff. */
+  if (thread)
+    g_thread_join (thread);
+
   if (ximagesink->ximage) {
     gst_buffer_unref (ximagesink->ximage);
     ximagesink->ximage = NULL;
