@@ -94,7 +94,7 @@ fail_if_audio (gpointer key, ChainState * state, gpointer data)
   fail_if (state->codec == CODEC_SPEEX, "speex BOS occurred before theora BOS");
 }
 
-static void
+static ChainState *
 validate_ogg_page (ogg_page * page)
 {
   gulong serialno;
@@ -146,8 +146,18 @@ validate_ogg_page (ogg_page * page)
         G_GINT64_FORMAT, serialno, state->last_granule, granule);
     state->last_granule = granule;
   }
+
+  return state;
 }
 
+static void
+is_video (gpointer key, ChainState * state, gpointer data)
+{
+  if (state->codec == CODEC_THEORA)
+    *((gboolean *) data) = TRUE;
+}
+
+
 static gboolean
 eos_buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
 {
@@ -155,6 +165,8 @@ eos_buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
   gint size;
   guint8 *data;
   gchar *oggbuffer;
+  ChainState *state = NULL;
+  gboolean has_video = FALSE;
 
   size = GST_BUFFER_SIZE (buffer);
   data = GST_BUFFER_DATA (buffer);
@@ -168,10 +180,24 @@ eos_buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
 
     ret = ogg_sync_pageout (&oggsync, &page);
     if (ret > 0)
-      validate_ogg_page (&page);
+      state = validate_ogg_page (&page);
   }
   while (ret != 0);
 
+  if (state) {
+    /* Now, we can do buffer-level checks...
+     * If we have video somewhere, then we should have DELTA_UNIT set on all
+     * non-header (not IN_CAPS), non-video buffers
+     */
+    g_hash_table_foreach (eos_chain_states, (GHFunc) is_video, &has_video);
+    if (has_video && state->codec != CODEC_THEORA) {
+      if (!GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_IN_CAPS))
+        fail_unless (GST_BUFFER_FLAG_IS_SET (buffer,
+                GST_BUFFER_FLAG_DELTA_UNIT),
+            "Non-video buffer doesn't have DELTA_UNIT in stream with video");
+    }
+  }
+
   return TRUE;
 }
 
@@ -302,6 +328,33 @@ GST_START_TEST (test_vorbis_theora)
 
 GST_END_TEST;
 
+GST_START_TEST (test_simple_cleanup)
+{
+  GstElement *oggmux;
+
+  oggmux = gst_element_factory_make ("oggmux", NULL);
+  gst_object_unref (oggmux);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_request_pad_cleanup)
+{
+  GstElement *oggmux;
+  GstPad *pad;
+
+  oggmux = gst_element_factory_make ("oggmux", NULL);
+  pad = gst_element_get_request_pad (oggmux, "sink_%d");
+  fail_unless (pad != NULL);
+  gst_object_unref (pad);
+  pad = gst_element_get_request_pad (oggmux, "sink_%d");
+  fail_unless (pad != NULL);
+  gst_object_unref (pad);
+  gst_object_unref (oggmux);
+}
+
+GST_END_TEST;
+
 static Suite *
 oggmux_suite (void)
 {
@@ -309,7 +362,6 @@ oggmux_suite (void)
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
-
 #ifdef HAVE_VORBIS
   tcase_add_test (tc_chain, test_vorbis);
 #endif
@@ -323,6 +375,8 @@ oggmux_suite (void)
   tcase_add_test (tc_chain, test_theora_vorbis);
 #endif
 
+  tcase_add_test (tc_chain, test_simple_cleanup);
+  tcase_add_test (tc_chain, test_request_pad_cleanup);
   return s;
 }
 
